SystemOrganization addCategory: #'SMark-Benchmarks-Classic'!
SystemOrganization addCategory: #'SMark-Benchmarks-DeltaBlue'!
SystemOrganization addCategory: #'SMark-Benchmarks-Richards'!
SystemOrganization addCategory: #'SMark-Benchmarks-SOM'!
SystemOrganization addCategory: #'SMark-Benchmarks-GraphSearch'!
SystemOrganization addCategory: #'SMark-Benchmarks-Json'!
SystemOrganization addCategory: #'SMark-Benchmarks-RSqueak'!
SystemOrganization addCategory: #'SMark-Benchmarks-SqueakJS'!
SystemOrganization addCategory: #'SMark-Benchmarks-NBody'!

Class subclass: #SMarkCompilerTargetClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkCompilerTargetClass methodsFor: 'compiling' stamp: ''!
acceptsLoggingOfCompilation
	"Answer whether the receiver's method submisions and class defintions should be logged to the changes file and to the current change set."

	^ false! !

!SMarkCompilerTargetClass methodsFor: 'accessing method dictionary' stamp: 'StefanMarr 2/13/2013 00:43'!
addAndClassifySelector: selector withMethod: compiledMethod inProtocol: aCategory notifying: requestor
	| priorMethodOrNil oldProtocol newProtocol |
	priorMethodOrNil := self compiledMethodAt: selector ifAbsent: [nil].
	self addSelectorSilently: selector withMethod: compiledMethod.
	oldProtocol := self organization categoryOfElement: selector.

	self classify: selector under: aCategory.

	newProtocol := self organization categoryOfElement: selector.! !

!SMarkCompilerTargetClass methodsFor: 'compatibility Pharo 1.4 vs. 2.0' stamp: 'StefanMarr 2/13/2013 00:42'!
classify: selector under: aCategory
	"Use image specific approach Pharo 1.4 vs. Pharo 2.0"
	| classify |
	classify := [self organization classify: selector under: aCategory].

	"Pharo 1.4"
	(Smalltalk at: #SystemChangeNotifier ifPresent: [:cls |
		^ cls uniqueInstance doSilently: classify ]).
	
	"Pharo 2.0"
	^ SystemAnnouncer uniqueInstance suspendAllWhile: classify.! !

!SMarkCompilerTargetClass methodsFor: 'initialization' stamp: 'sm 1/17/2011 09:38'!
initialize
	super initialize.
	self methodDictionary: MethodDictionary new.! !

Object subclass: #SMarkAbstractConstraint
	instanceVariableNames: 'strength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkAbstractConstraint commentStamp: '<historical>' prior: 0!
I am an abstract class representing a system-maintainable relationship (or "constraint") between a set of variables. I supply a strength instance variable; concrete subclasses provide a means of storing the constrained variables and other information required to represent a constraint.

Instance variables:
	strength			the strength of this constraint <Strength>!

!SMarkAbstractConstraint methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:08'!
addConstraint
	"Activate this constraint and attempt to satisfy it."

	self addToGraph.
	SMarkPlanner current incrementalAdd: self.! !

!SMarkAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
addToGraph
	"Add myself to the constraint graph."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision. The output of
	 the choosen method must not have the given mark and must have a
	 walkabout strength less than that of this constraint."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:08'!
destroyConstraint
	"Deactivate this constraint, remove it from the constraint graph,
	 possibly causing other constraints to be satisfied, and destroy it."

	(self isSatisfied) ifTrue: [SMarkPlanner current incrementalRemove: self].
	self removeFromGraph.
	"self release."! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
inputsDo: aBlock
	"Assume that I am satisfied. Evaluate the given block on all my current
	 input variables."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 14:55'!
inputsKnown: mark
	"Assume that I am satisfied. Answer true if all my current inputs are
	 known. A variable is known if either a) it is 'stay' (i.e. it is a
	 constant at plan execution time), b) it has the given mark (indicating
	 that it has been computed by a constraint appearing earlier in the
	 plan), or c) it is not determined by any constraint."

	self inputsDo:
		[ :v |
		 (v mark = mark or: [(v stay) or: [v determinedBy == nil]]) ifFalse:
			[^false]].
	^true! !

!SMarkAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:53'!
isInput
	"Normal constraints are not input constraints. An input constraint is
	 one that depends on external state, such as the mouse, the keyboard,
	 a clock, or some arbitrary piece of imperative code."

	^false! !

!SMarkAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:54'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
longPrintOn: aStream

	aStream nextPut: $(.
	self shortPrintOn: aStream.
	aStream space; nextPutAll: self strength printString.
	(self isSatisfied)
		ifTrue:
			[aStream cr; space; space; space.
			 self inputsDo:
				[ :in | aStream nextPutAll: 'v', in printString, ' '].
			aStream nextPutAll: '-> '.
			aStream nextPutAll: 'v', self output printString]
		ifFalse:
			[aStream space; nextPutAll: 'UNSATISFIED'].
	aStream nextPut: $); cr.! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
output
	"Answer my current output variable. Raise an error if I am not
	 currently satisfied."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:57'!
printOn: aStream

	self shortPrintOn: aStream! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:56'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
removeFromGraph
	"Remove myself from the constraint graph."

	self subclassResponsibility! !

!SMarkAbstractConstraint methodsFor: 'planning' stamp: 'tfel 5/15/2015 11:09'!
satisfy: mark
	"Attempt to find a way to enforce this (still unsatisfied) constraint.
	 If successful, record the solution, perhaps modifying the current
	 dataflow graph.  Answer the constraint that this constraint overrides,
	 if there is one, or nil, if there isn't."

	| overridden out |
	self chooseMethod: mark.
	(self isSatisfied)
		ifTrue:			"constraint can be satisfied"
			["mark inputs to allow cycle detection in addPropagate"
			 self inputsDo: [ :in | in mark: mark].
			 out := self output.
			 overridden := out determinedBy.
			 (overridden == nil) ifFalse: [overridden markUnsatisfied].
			 out determinedBy: self.
			 (SMarkPlanner current addPropagate: self mark: mark) ifFalse:
				[self notify:
					('Cycle encountered adding:\   ',
					 self printString,
					 '\Constraint removed.') withCRs.
				 ^nil].
			 out mark: mark]
		ifFalse:			"constraint cannot be satisfied"
			[overridden := nil.
			 (strength sameAs: (SMarkPlanner required)) ifTrue:
				[self notify: 'Failed to satisfy a required constraint']].
	^ overridden! !

!SMarkAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
shortPrintOn: aStream

	aStream nextPutAll: self class name, '(', self printString, ')'.! !

!SMarkAbstractConstraint methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:53'!
strength
	^ strength! !

!SMarkAbstractConstraint methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:09'!
strength: strengthSymbol
	strength := SMarkPlanner of: strengthSymbol! !

SMarkAbstractConstraint subclass: #SMarkBinaryConstraint
	instanceVariableNames: 'direction v1 v2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkBinaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having two possible output variables.

Instance variables:
	v1, v2		possible output variables <Variable>
	direction		one of:
					#forward (v2 is output)
					#backward (	v1 is output)
					nil (not satisfied)!

!SMarkBinaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:21'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
	v2 addConstraint: self.
	direction := nil.! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
chooseMethod: mark
	"Decide if I can be satisfied and which way I should flow based on the relative strength of the variables I relate, and record that decision."

	(v1 mark == mark) ifTrue:		"forward or nothing"
		[((v2 mark ~= mark) and: [strength stronger: v2 walkStrength]) 
			ifTrue: [ ^ direction := #forward ]
			ifFalse: [ ^ direction := nil ] ].

	(v2 mark == mark) ifTrue:		"backward or nothing"
		[((v1 mark ~= mark) and: [strength stronger: v1 walkStrength])
			ifTrue: [ ^ direction := #backward ]
			ifFalse: [ ^ direction := nil ] ].

	"if we get here, neither variable is marked, so we have choice"
	(v1 walkStrength weaker: v2 walkStrength)
		ifTrue:
			[(strength stronger: v1 walkStrength)
				ifTrue: [ ^ direction := #backward ]
				ifFalse: [ ^ direction := nil ]]
		ifFalse:
			[(strength stronger: v2 walkStrength)
				ifTrue: [ ^ direction := #forward ]
				ifFalse: [ ^ direction := nil ]]. ! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	direction == #forward
		ifTrue: [ aBlock value: v1 ]
		ifFalse: [ aBlock value: v2 ].! !

!SMarkBinaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:21'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^ direction notNil! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	direction := nil.! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
output
	"Answer my current output variable."

	direction == #forward
		ifTrue: [ ^ v2 ]
		ifFalse: [ ^ v1 ]! !

!SMarkBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:02'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
         the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [in := v2. out := v1].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: (in stay).
	(out stay) ifTrue: [ self execute ].		"stay optimization"! !

!SMarkBinaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 14:58'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	direction := nil.! !

!SMarkBinaryConstraint methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:08'!
var: variable1 var: variable2 strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := SMarkPlanner of: strengthSymbol.
	v1 := variable1.
	v2 := variable2.
	direction := nil.
	self addConstraint.! !

SMarkBinaryConstraint subclass: #SMarkEqualityConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkEqualityConstraint commentStamp: '<historical>' prior: 0!
I constrain two variables to have the same value: "v1 = v2".!

!SMarkEqualityConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:55'!
var: variable1 var: variable2 strength: strengthSymbol
	"Install a constraint with the given strength equating the given
	 variables."

	^(self new) var: variable1 var: variable2 strength: strengthSymbol! !

!SMarkEqualityConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:18'!
execute
	"Enforce this constraint. Assume that it is satisfied."
 
	(direction == #forward)
		ifTrue: [v2 value: v1 value]
		ifFalse: [v1 value: v2 value].! !

SMarkBinaryConstraint subclass: #SMarkScaleConstraint
	instanceVariableNames: 'offset scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkScaleConstraint commentStamp: '<historical>' prior: 0!
I relate two variables by the linear scaling relationship:
"v2 = (v1 * scale) + offset". Either v1 or v2 may be changed to maintain this
relationship but the scale factor and offset are considered read-only.

Instance variables:
	scale		scale factor input variable <Variable>
	offset		offset input variable <Variable>!

!SMarkScaleConstraint class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:32'!
var: src var: scale var: offset var: dst strength: strengthSymbol
	"Install a scale constraint with the given strength on the given variables."

	^(self new) src: src scale: scale offset: offset dst: dst strength: strengthSymbol! !

!SMarkScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:23'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
   v2 addConstraint: self.
   scale addConstraint: self.
   offset addConstraint: self.
   direction := nil.
! !

!SMarkScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:30'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	(direction == #forward)
		ifTrue: [v2 value: (v1 value * scale value) + offset value]
		ifFalse: [v1 value: (v2 value - offset value) // scale value].! !

!SMarkScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	(direction == #forward)
		ifTrue: [ aBlock value: v1.
					 aBlock value: scale.
					 aBlock value: offset]
		ifFalse: [ aBlock value: v2.
					  aBlock 	value: scale. 
					  aBlock 	value: offset].! !

!SMarkScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay', the value for the current output of this constraint. Assume this constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [out := v1. in := v2].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: ((in stay) and: [(scale stay) and: [offset stay]]).
	out stay ifTrue: [self execute].		"stay optimization"! !

!SMarkScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:24'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	(scale == nil) ifFalse: [scale removeConstraint: self].
	(offset == nil) ifFalse: [offset removeConstraint: self].
	direction := nil.
! !

!SMarkScaleConstraint methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:08'!
src: srcVar scale: scaleVar offset: offsetVar dst: dstVar strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := SMarkPlanner of: strengthSymbol.
	v1 := srcVar.
	v2 := dstVar.
	scale := scaleVar.
	offset := offsetVar.
	direction := nil.
	self addConstraint.! !

SMarkAbstractConstraint subclass: #SMarkUnaryConstraint
	instanceVariableNames: 'output satisfied'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkUnaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having a single possible output variable.

Instance variables:
	output		possible output variable <Variable>
	satisfied		true if I am currently satisfied <Boolean>!

SMarkUnaryConstraint subclass: #SMarkEditConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkEditConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same. I have one method with zero inputs and one output, which does nothing. Planners may exploit the fact that, if I am satisfied, my output will not change during plan execution. This is called "stay optimization."!

!SMarkEditConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:16'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!SMarkEditConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
execute
	"Edit constraints do nothing."! !

!SMarkEditConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:18'!
isInput
	"I indicate that a variable is to be changed by imperative code."

	^true! !

SMarkUnaryConstraint subclass: #SMarkStayConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkStayConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same.
I have one method with zero inputs and one output, which does nothing. Planners
may exploit the fact that, if I am satisfied, my output will not change during
plan execution. This is called "stay optimization."!

!SMarkStayConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:17'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!SMarkStayConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:20'!
execute
	"Stay constraints do nothing."! !

!SMarkUnaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:16'!
addToGraph
	"Add myself to the constraint graph."

	output addConstraint: self.
	satisfied := false.! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision."

	satisfied :=
		output mark ~= mark and:
		[strength stronger: output walkStrength].! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
inputsDo: aBlock
	"I have no input variables."! !

!SMarkUnaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:15'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^satisfied! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	satisfied := false.! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
output
	"Answer my current output variable."

	^ output! !

!SMarkUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	output walkStrength: strength.
	output stay: self isInput not.
	output stay ifTrue: [self execute].	"stay optimization"! !

!SMarkUnaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:41'!
removeFromGraph
	"Remove myself from the constraint graph."

	(output == nil) ifFalse: [output removeConstraint: self].
	satisfied := false.! !

!SMarkUnaryConstraint methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:08'!
var: aVariable strength: strengthSymbol
	"Initialize myself with the given variable and strength."

	strength := SMarkPlanner of: strengthSymbol.
	output := aVariable.
	satisfied := false.
	self addConstraint.! !

Object subclass: #SMarkGSEdge
	instanceVariableNames: 'dest weight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!SMarkGSEdge class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:25'!
newWith: dest and: weight
	^ self basicNew initializeWith: dest and: weight! !

!SMarkGSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:24'!
dest
	^ dest! !

!SMarkGSEdge methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:25'!
initializeWith: destination and: w
	dest   := destination.
	weight := w.! !

!SMarkGSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:25'!
weight
	^ weight! !

Object subclass: #SMarkGSJenkinsRandom
	instanceVariableNames: ''
	classVariableNames: 'seed'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!SMarkGSJenkinsRandom methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:19'!
random
	"Robert Jenkins' 32 bit integer hash function."
	 seed := ((seed       + 2127912214 "0x7ed55d16")       + (seed as32BitUnsignedValue  << 12) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed bitXor: 3345072700 "0xc761c23c") bitXor: (seed as32BitUnsignedValue >>> 19)) as32BitSignedValue.
    seed := ((seed       +  374761393 "0x165667B1")       + (seed as32BitUnsignedValue  <<  5) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed       + 3550635116 "0xd3a2646c") bitXor: (seed as32BitUnsignedValue  <<  9) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed       + 4251993797 "0xfd7046c5")       + (seed as32BitUnsignedValue  <<  3) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed bitXor: 3042594569 "0xb55a4f09") bitXor: (seed as32BitUnsignedValue >>> 16)) as32BitSignedValue.
    ^ seed! !

!SMarkGSJenkinsRandom methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:37'!
seed: val
	seed := ThirtyTwoBitRegister basicNew load: val! !

Object subclass: #SMarkGSNode
	instanceVariableNames: 'starting noOfEdges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!SMarkGSNode class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:24'!
newWith: starting and: noOfEdges
	^ self basicNew initializeWith: starting and: noOfEdges! !

!SMarkGSNode methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:23'!
initializeWith: start and: edges
	starting := start.
	noOfEdges:= edges! !

!SMarkGSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
noOfEdges
	^ noOfEdges! !

!SMarkGSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
starting
	^ starting! !

Object subclass: #SMarkJSHashIndexTable
	instanceVariableNames: 'hashTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:22'!
at: name
    | slot |
    slot := self hashSlotFor: name.

    " subtract 1, 0 stands for empty "
    ^ ((hashTable at: slot) & 255) - 1! !

!SMarkJSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:16'!
at: name put: index
    | slot |
    slot := self hashSlotFor: name.
    
    index < 255
      ifTrue:  [ hashTable at: slot put: index + 1 ]
      ifFalse: [ hashTable at: slot put: 0 ]! !

!SMarkJSHashIndexTable methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 13:06'!
hashSlotFor: element
    ^ (element hash & 31) + 1! !

!SMarkJSHashIndexTable methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:16'!
initialize
	hashTable := IntegerArray new: 32 withAll: 0! !

!SMarkJSHashIndexTable methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:21'!
remove: index
    1 to: hashTable size do: [:i |
      index + 1 = (hashTable at: i)
        ifTrue:  [ hashTable at: i put: 0 ]
        ifFalse: [ 
          (hashTable at: i) > index + 1 ifTrue: [
            hashTable at: i put: (hashTable at: i) - 1]]]! !

Object subclass: #SMarkJSJsonParser
	instanceVariableNames: 'input index line column current captureBuffer captureStart exceptionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonParser class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 11:38'!
with: aJsonString
    ^ self basicNew initializeWith: aJsonString! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:03'!
endCapture
    | captured |
    '' = captureBuffer
      ifTrue:  [ captured := input copyFrom: captureStart to: index - 1 ]
      ifFalse: [
        self pauseCapture.
        captured := captureBuffer.
        captureBuffer := '' ].
    captureStart := -1.
    
    ^ captured! !

!SMarkJSJsonParser methodsFor: 'error handling' stamp: 'tfel 5/15/2015 11:37'!
error: message
    'error:' print.
    message print.
    ':' print.
    line print.
    column println.
    exceptionBlock value: (SMarkJSParseException with: message at: index
                                          line: line    column: column )
  ! !

!SMarkJSJsonParser methodsFor: 'error handling' stamp: 'StefanMarr 4/3/2015 11:45'!
expected: expected
    self isEndOfText ifTrue: [
      self error: 'Unexpected end of input, expected ', expected asString
    ].
    self error: 'Expected ', expected! !

!SMarkJSJsonParser methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:31'!
initializeWith: string
    input := string.
    index := 0.
    line  := 1.
    column := 0.
    current := nil.
    captureBuffer := ''.! !

!SMarkJSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isDigit
    current = '0' ifTrue: [^ true].
    current = '1' ifTrue: [^ true].
    current = '2' ifTrue: [^ true].
    current = '3' ifTrue: [^ true].
    current = '4' ifTrue: [^ true].
    current = '5' ifTrue: [^ true].
    current = '6' ifTrue: [^ true].
    current = '7' ifTrue: [^ true].
    current = '8' ifTrue: [^ true].
    current = '9' ifTrue: [^ true].
    ^ false! !

!SMarkJSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isEndOfText
    ^ current isNil! !

!SMarkJSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isWhiteSpace
    current = ' '  ifTrue: [^ true].
    current = '\t' ifTrue: [^ true].
    current = '\n' ifTrue: [^ true].
    current = '\r' ifTrue: [^ true].
    ^ false! !

!SMarkJSJsonParser methodsFor: 'parsing' stamp: 'StefanMarr 4/3/2015 11:31'!
parse
    | result |
    exceptionBlock := [:ex | ^ ex ].
    self read.
    self skipWhiteSpace.
    result := self readValue.
    self skipWhiteSpace.
    self isEndOfText ifFalse: [ self error: 'Unexpected character'].
    ^ result! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:37'!
pauseCapture
    captureBuffer := captureBuffer concatenate: (
      input substringFrom: captureStart to: index - 1).
    captureStart := -1! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:02'!
read
	"TODO: this is probably broken"
    current = '\n' ifTrue: [
      line := line + 1.
      column := 0.
    ].
    
    index := index + 1.
    column := column + 1.

    input ifNil: [ self error:'input nil'].
    index <= input size
      ifTrue:  [ current := input copyFrom: index to: index ]
      ifFalse: [ current := nil ]! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:36'!
readArray
    | array |
    self read.
    array := SMarkJSJsonArray new.
    
    "Array might be empty"
    self skipWhiteSpace.
    (self readChar: ']') ifTrue: [
      ^ array
    ].
    
    self readArrayElement: array.
    [self readChar: ','] whileTrue: [
      self readArrayElement: array.
    ].

    (self readChar: ']') ifFalse: [
      self expected: '"," or "]"'
    ].
    ^ array! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readArrayElement: array
    self skipWhiteSpace.
    array add: self readValue.
    self skipWhiteSpace.! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readChar: ch
    current = ch ifFalse: [ ^ false ].
    self read.
    ^ true! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readDigit
    self isDigit ifFalse: [ ^ false ].
    self read.
    ^ true! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscape
    self read.    
    captureBuffer := captureBuffer concatenate: self readEscapeChar.
    self read! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscapeChar
    current = '"'  ifTrue: [ ^ '"'  ].
    current = '/'  ifTrue: [ ^ '/'  ].
    current = '\\' ifTrue: [ ^ '\\' ].

    current = 'b'  ifTrue: [ ^ '\b'  ].
    current = 'f'  ifTrue: [ ^ '\f'  ].
    current = 'n'  ifTrue: [ ^ '\n'  ].
    current = 'r'  ifTrue: [ ^ '\r'  ].
    current = 't'  ifTrue: [ ^ '\t'  ].

    "TODO: SOM doesn't have a way to create unicode characters."
    self expected: 'valid escape sequence. note, some are not supported'
! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readExponent
    ((self readChar: 'e') not and: [
      (self readChar: 'E') not]) ifTrue: [ ^ false ].
    
    
    (self readChar: '+') ifFalse: [ self readChar: '-' ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].

    [self readDigit] whileTrue: [].

    ^ true! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:36'!
readFalse
    self read.
    self readRequiredChar: 'a'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 's'.
    self readRequiredChar: 'e'.
    ^ SMarkJSJsonLiteral FALSE! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readFraction
    (self readChar: '.') ifFalse: [ ^ false ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    
    [self readDigit] whileTrue: [].

    ^ true! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readName
    current = '"' ifFalse: [ self expected: 'name' ].
    ^ self readStringInternal! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:36'!
readNull
    self read.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 'l'.
    ^ SMarkJSJsonLiteral NULL! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:36'!
readNumber
    | firstDigit |
    self startCapture.
    self readChar: '-'.
    firstDigit := current.
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    firstDigit ~= '0' ifTrue: [ [self readDigit] whileTrue: []].
        
    self readFraction.
    self readExponent.
    ^ SMarkJSJsonNumber new: self endCapture! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:37'!
readObject
    | object |
    self read.
    object := SMarkJSJsonObject new.
    self skipWhiteSpace.

    (self readChar: '}') ifTrue: [
      ^ object
    ].
    
    self readObjectKeyValuePair: object.
    [self readChar: ','] whileTrue: [
      self readObjectKeyValuePair: object.
    ].
    
    (self readChar: '}') ifFalse: [
      self expected: '"," or "}"'
    ].
    
    ^ object! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readObjectKeyValuePair: object
    | name |
    self skipWhiteSpace.
    name := self readName.
    self skipWhiteSpace.
  
    (self readChar: ':') ifFalse: [ self expected: ':' ].
  
    self skipWhiteSpace.
  
    object add: name with: self readValue.
  
    self skipWhiteSpace! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readRequiredChar: ch
    (self readChar: ch) ifFalse: [
      self expected: 'character: ' + ch
    ]! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:37'!
readString
    ^ SMarkJSJsonString new: self readStringInternal! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readStringInternal
    | string |
    self read.
    self startCapture.
    
    [current = '"'] whileFalse: [
      current = '\\' ifTrue: [
        self pauseCapture.
        self readEscape.
        self startCapture.
      ] ifFalse: [
        "if (current < 0x20) { throw expected('valid string character'); }"
        "we currently don't have a way to get the ordinal value for a character"
        "} else {"
        self read.
      ]
    ].
    string := self endCapture.
    self read.
    ^ string! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'tfel 5/15/2015 11:36'!
readTrue
    self read.
    self readRequiredChar: 'r'.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'e'.
    ^ SMarkJSJsonLiteral TRUE! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:31'!
readValue
    current = 'n' ifTrue: [ ^ self readNull   ].
    current = 't' ifTrue: [ ^ self readTrue   ].
    current = 'f' ifTrue: [ ^ self readFalse  ].
    current = '"' ifTrue: [ ^ self readString ].
    current = '[' ifTrue: [ ^ self readArray  ].
    current = '{' ifTrue: [ ^ self readObject ].
    
    "Is this really the best way to write this?, or better #or:?,
     but with all the nesting, it's just ugly."
    current = '-' ifTrue: [ ^ self readNumber ].
    current = '0' ifTrue: [ ^ self readNumber ].
    current = '1' ifTrue: [ ^ self readNumber ].
    current = '2' ifTrue: [ ^ self readNumber ].
    current = '3' ifTrue: [ ^ self readNumber ].
    current = '4' ifTrue: [ ^ self readNumber ].
    current = '5' ifTrue: [ ^ self readNumber ].
    current = '6' ifTrue: [ ^ self readNumber ].
    current = '7' ifTrue: [ ^ self readNumber ].
    current = '8' ifTrue: [ ^ self readNumber ].
    current = '9' ifTrue: [ ^ self readNumber ].
    
    "else"
    self expected: 'value'! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
skipWhiteSpace
    [ self isWhiteSpace ]
      whileTrue:
        [ self read ].! !

!SMarkJSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
startCapture
    captureStart := index! !

Object subclass: #SMarkJSJsonValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

SMarkJSJsonValue subclass: #SMarkJSJsonArray
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonArray class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:35'!
readFrom: string
    ^ (SMarkJSJsonValue readFrom: string) asArray! !

!SMarkJSJsonArray methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:51'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ values = other values! !

!SMarkJSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:49'!
add: value
    value ifNil: [ self error: 'value is null' ].
    values add: value! !

!SMarkJSJsonArray methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:35'!
addBoolean: value
    values add: (SMarkJSJsonValue boolean: value)! !

!SMarkJSJsonArray methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:35'!
addDouble: value
	values add: (SMarkJSJsonValue double: value)! !

!SMarkJSJsonArray methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:35'!
addInteger: value
    values add: (SMarkJSJsonValue integer: value)! !

!SMarkJSJsonArray methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:35'!
addString: value
    values add: (SMarkJSJsonValue string: value)! !

!SMarkJSJsonArray methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:51'!
asArray
	^ self! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index
	^ values at: index! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index put: value
    value ifNil: [ self error: 'value is null' ].
    values at: index put: value! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:35'!
at: index putBoolean: value
    values at: index put: (SMarkJSJsonValue boolean: value)! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:35'!
at: index putDouble: value
    values at: index put: (SMarkJSJsonValue double: value)! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:35'!
at: index putInteger: value
    values at: index put: (SMarkJSJsonValue integer: value)! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:35'!
at: index putString: value
    values at: index put: (SMarkJSJsonValue string: value)! !

!SMarkJSJsonArray methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:51'!
hashcode
	^ values hash! !

!SMarkJSJsonArray methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:47'!
initialize
	values := OrderedCollection new.! !

!SMarkJSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:51'!
isArray
	^ true! !

!SMarkJSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:50'!
isEmpty
    ^ values isEmpty! !

!SMarkJSJsonArray methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:50'!
removeAt: index
    values removeAt: index! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
size
    ^ values size! !

!SMarkJSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:51'!
values
	^ values! !

SMarkJSJsonValue subclass: #SMarkJSJsonLiteral
	instanceVariableNames: 'value isNull isTrue isFalse'
	classVariableNames: 'FALSE NULL TRUE'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
FALSE
	^ FALSE! !

!SMarkJSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
NULL
	^ NULL! !

!SMarkJSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
TRUE
	^ TRUE! !

!SMarkJSJsonLiteral class methodsFor: 'class initialization' stamp: 'StefanMarr 4/3/2015 13:13'!
initialize
   "self initialize"
	NULL := self basicNew initializeWith: 'null'.
	TRUE := self basicNew initializeWith: 'true'.
	FALSE:= self basicNew initializeWith: 'false'.! !

!SMarkJSJsonLiteral methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:50'!
= other
    self == other ifTrue: [ ^ true ].  
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ value = other asString
! !

!SMarkJSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:49'!
asBoolean
	isNull ifTrue: [ ^ super asBoolean ]
	       ifFalse:[ ^ isTrue ]! !

!SMarkJSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:44'!
asString
	^ value! !

!SMarkJSJsonLiteral methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:44'!
hashcode
	^ value hash! !

!SMarkJSJsonLiteral methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:44'!
initializeWith: val
    value   := val.
    isNull  := 'null'  = val.
    isTrue  := 'true'  = val.
    isFalse := 'false' = val.! !

!SMarkJSJsonLiteral methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:49'!
isBoolean
	^ isTrue | isFalse! !

!SMarkJSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isFalse
	^ isFalse! !

!SMarkJSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isNull
	^ isNull! !

!SMarkJSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isTrue
	^ isTrue! !

SMarkJSJsonValue subclass: #SMarkJSJsonNumber
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonNumber class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 11:05'!
new: string
    string ifNil: [ self error: 'string is null' ].
    ^ self new initializeWith: string
! !

!SMarkJSJsonNumber methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:58'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    
    self class == other class ifFalse: [ ^ false ].
    ^ string = other asString
! !

!SMarkJSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:57'!
asDouble
	^ Float readFromString: string! !

!SMarkJSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:53'!
asInteger
	^ string asInteger! !

!SMarkJSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:52'!
asString
	^ string! !

!SMarkJSJsonNumber methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:58'!
hashcode
	^ string hash! !

!SMarkJSJsonNumber methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:52'!
initializeWith: str
	string := str! !

!SMarkJSJsonNumber methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:52'!
isNumber
	^ true! !

SMarkJSJsonValue subclass: #SMarkJSJsonObject
	instanceVariableNames: 'names values table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonObject class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:36'!
readFrom: string
	^ (SMarkJSJsonValue readFrom: string) asObject! !

!SMarkJSJsonObject methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 11:16'!
= other
    self == other ifTrue: [ ^ true  ].
    self == nil   ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].
    
    ^ names = other names & values = other values! !

!SMarkJSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 13:05'!
add: name with: aJsonValue
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].

    names  add: name.
    values add: aJsonValue.
    table at: name put: names size.! !

!SMarkJSJsonObject methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:36'!
add: name withBoolean: value
    self add: name with: (SMarkJSJsonValue boolean: value)! !

!SMarkJSJsonObject methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:36'!
add: name withNumber: value
    self add: name with: (SMarkJSJsonValue number: value)! !

!SMarkJSJsonObject methodsFor: 'adding' stamp: 'tfel 5/15/2015 11:36'!
add: name withString: value
    self add: name with: (SMarkJSJsonValue string: value)! !

!SMarkJSJsonObject methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:15'!
asObject
	^ self! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    idx := self indexOf: name.
    idx = -1
      ifTrue:  [ ^ nil ]
      ifFalse: [ ^ values at: idx ]! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asBooleanWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asBoolean ]! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asDoubleWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asDouble ]! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asIntegerWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asInteger ]! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asStringWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asString ]! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:12'!
at: name put: aJsonValue
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].
  
    idx := self indexOf: name.
    idx <> -1
      ifTrue:  [values at: idx put: aJsonValue]
      ifFalse: [
        names  append: name.
        values append: aJsonValue.
        table at: name put: names size.
      ].! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:36'!
at: name putBoolean: value
    self at: name put: (SMarkJSJsonValue boolean: value)! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:36'!
at: name putNumber: value
    self at: name put: (SMarkJSJsonValue number: value)! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:36'!
at: name putString: value
    self at: name put: (SMarkJSJsonValue string: value)! !

!SMarkJSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:15'!
hashcode
	| result |
    result := 1.
    result := 31 * result + names hash.
    result := 31 * result + values hash.
    ^ result
! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:15'!
indexOf: name
    | idx |
    idx := table at: name.    
    idx ~= -1 & name = (names at: idx) ifTrue: [ ^ idx ].
    ^ names lastIndexOf: name! !

!SMarkJSJsonObject methodsFor: 'initialization' stamp: 'tfel 5/15/2015 11:35'!
initialize
    names  := OrderedCollection new.
    values := OrderedCollection new.
    table  := SMarkJSHashIndexTable new
! !

!SMarkJSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isEmpty
	^ names isEmpty! !

!SMarkJSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isObject
	^ true! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
names
	^ names! !

!SMarkJSJsonObject methodsFor: 'removing' stamp: 'StefanMarr 4/3/2015 11:14'!
remove: name
    | idx |
    name ifNil: [ self error: 'name is null' ].
    idx := self indexOf: name.
    idx = -1 ifFalse: [
      table  remove: name.
      names  remove: idx.
      values remove: idx.
    ].! !

!SMarkJSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
size
	^ names size! !

!SMarkJSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:17'!
updateHashIndex
    | size |
    size := names size.
    1 to: size do: [: i |
      table add: (names at: i) put: i
    ]! !

SMarkJSJsonValue subclass: #SMarkJSJsonString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJsonString class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:33'!
new: str
	^ self basicNew initializeWith: str! !

!SMarkJSJsonString methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:35'!
= other
	self == other ifTrue: [ ^ true ].
	other == nil ifTrue: [ ^ false ].

	self class == other class ifFalse: [ ^ false ].

	^ string =  other asString! !

!SMarkJSJsonString methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:34'!
asString
	^ string! !

!SMarkJSJsonString methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:34'!
hashcode
	^ string hash! !

!SMarkJSJsonString methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:34'!
initializeWith: str
	string := str! !

!SMarkJSJsonString methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:34'!
isString
	^ true! !

!SMarkJSJsonValue class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:36'!
boolean: aBoolean
	^ aBoolean
		ifTrue:  [ SMarkJSJsonLiteral TRUE ]
		ifFalse: [ SMarkJSJsonLiteral FALSE ]! !

!SMarkJSJsonValue class methodsFor: 'helper' stamp: 'StefanMarr 4/2/2015 22:32'!
cutOffPointZero: str
    (str endsWith: '.0') ifTrue: [ ^ str substringFrom: 1 to: str length - 2].
    ^ str! !

!SMarkJSJsonValue class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:36'!
double: aDouble
	aDouble ifNil: [ self error: 'aDouble nil' ].
	^ SMarkJSJsonNumber new: (self cutOffPointZero: aDouble asString)! !

!SMarkJSJsonValue class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:36'!
integer: anInteger
	^ SMarkJSJsonNumber new: anInteger asString! !

!SMarkJSJsonValue class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:35'!
readFrom: str 
	^ (SMarkJSJsonParser with: str) parse! !

!SMarkJSJsonValue class methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:37'!
string: aString
	^ aString == nil
		ifTrue:  [ SMarkJSJsonLiteral NULL ]
		ifFalse: [ SMarkJSJsonString new: aString ]! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asArray
	self error: 'Unsupported operation, not an array ' + self asString! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asBoolean
	self error: 'Unsupported operation, not a boolean ' + self asString! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asDouble
	self error: 'Unsupported operation, not a double ' + self asString! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asInteger
	self error: 'Unsupported operation, not an integer ' + self asString! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asObject
	self error: 'Unsupported operation, not an object: ' + self asString! !

!SMarkJSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asString
	self error: 'Unsupported operation, not a string ' + self asString! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isArray
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isBoolean
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isFalse
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isNull
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isNumber
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isObject
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isString
	^ false! !

!SMarkJSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isTrue
	^ false! !

!SMarkJSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeOn: writer
	^ self subclassResponsibility! !

!SMarkJSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeTo: writer
	self writeTo: writer with: nil! !

Object subclass: #SMarkJSParseException
	instanceVariableNames: 'msg offset line column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSParseException methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:41'!
asString ^ msg, ':', line, ':', column! !

!SMarkJSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
column ^ column! !

!SMarkJSParseException methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:40'!
initializeWith: message at: anOffset line: aLine column: aColumn
    msg    := message.
    offset := anOffset.
    line   := aLine.
    column := aColumn.! !

!SMarkJSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
line ^ line! !

!SMarkJSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
message ^ msg! !

!SMarkJSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
offset ^ offset! !

Object subclass: #SMarkNBBody
	instanceVariableNames: 'x y z vx vy vz mass'
	classVariableNames: 'SolarMass'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
DaysPerYear
	^ 365.24! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
Pi
	^ 3.141592653589793! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:47'!
SolarMass
	^ SolarMass! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:47'!
initialize
	"self initialize"
	SolarMass := 4 * self Pi * self Pi.
	! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
jupiter
		| b |
       b := super new.
       b x:    4.8414314424647209.
       b y:   -1.16032004402742839.
       b z:   -0.103622044471123109.
       b vx:   0.00166007664274403694   * self DaysPerYear.
       b vy:   0.00769901118419740425   * self DaysPerYear.
       b vz:  -0.0000690460016972063023 * self DaysPerYear.
       b mass: 0.000954791938424326609  * self SolarMass.
       ^ b! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
neptune
		| b |
       b := super new.
       b x:   15.3796971148509165.
       b y:  -25.9193146099879641.
       b z:    0.179258772950371181.
       b vx:   0.00268067772490389322   * self DaysPerYear.
       b vy:   0.00162824170038242295   * self DaysPerYear.
       b vz:  -0.000095159225451971587  * self DaysPerYear.
       b mass: 0.0000515138902046611451 * self SolarMass.
       ^ b! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
new
	| b |
        b := super new.
        b x: 0.0.  b vx: 0.0.
        b y: 0.0.  b vy: 0.0.
        b z: 0.0.  b vz: 0.0.
        b mass: 0.0.
        ^ b! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
saturn 
		| b |
       b := super new.
       b x:    8.34336671824457987.
       b y:    4.12479856412430479.
       b z:   -0.403523417114321381.
       b vx:  -0.00276742510726862411   * self DaysPerYear.
       b vy:   0.00499852801234917238   * self DaysPerYear.
       b vz:   0.0000230417297573763929 * self DaysPerYear.
       b mass: 0.000285885980666130812  * self SolarMass.
       ^ b! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
sun 
		| b |
       b := self new.
       b mass: self SolarMass.
       ^ b! !

!SMarkNBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
uranus
		| b |
       b := super new.
       b x:   12.894369562139131.
       b y:  -15.1111514016986312.
       b z:   -0.223307578892655734.
       b vx:   0.00296460137564761618   * self DaysPerYear.
       b vy:   0.0023784717395948095    * self DaysPerYear.
       b vz:  -0.0000296589568540237556 * self DaysPerYear.
       b mass: 0.0000436624404335156298 * self SolarMass.
       ^ b! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
mass ^ mass! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
mass: val
	mass := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 11:29'!
offsetMomentumX: px y: py z: pz
       vx := 0.0 - (px / SMarkNBBody SolarMass).
       vy := 0.0 - (py / SMarkNBBody SolarMass).
       vz := 0.0 - (pz / SMarkNBBody SolarMass).! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
vx ^ vx! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vx: val
	vx := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy ^ vy! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy: val
	vy := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vz ^ vz! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
vz: val
	vz := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
x ^ x! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
x: val
	x := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
y ^ y! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
y: val
	y := val! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
z ^ z! !

!SMarkNBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
z: val
	z := val! !

Object subclass: #SMarkNBodySystem
	instanceVariableNames: 'bodies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!SMarkNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
advance: dt
        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.

            i + 1 to: bodies size do: [:j |
                | dx dy dz jBody dSquared distance mag |
                jBody := bodies at: j.
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                dSquared := (dx * dx) + (dy * dy) + (dz * dz).
                distance := dSquared sqrt.
                mag      := dt / (dSquared * distance).

                iBody vx: iBody vx - (dx * jBody mass * mag).
                iBody vy: iBody vy - (dy * jBody mass * mag).
                iBody vz: iBody vz - (dz * jBody mass * mag).

                jBody vx: jBody vx + (dx * iBody mass * mag).
                jBody vy: jBody vy + (dy * iBody mass * mag).
                jBody vz: jBody vz + (dz * iBody mass * mag).
            ].
       ].
       
       bodies do: [:body |
           body x: body x + (dt * body vx).
           body y: body y + (dt * body vy).
           body z: body z + (dt * body vz).
       ].! !

!SMarkNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
energy
        | dx dy dz distance e |
        e := 0.0.

        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.
            
            e := e + (0.5 * iBody mass *
                 ((iBody vx * iBody vx) +
                  (iBody vy * iBody vy) +
                  (iBody vz * iBody vz))).
            
            i + 1 to: bodies size do: [:j |
                | jBody |
                jBody := bodies at: j.
                
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
                e := e - ((iBody mass * jBody mass) / distance).
            ].
       ].
       ^ e! !

!SMarkNBodySystem methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 11:29'!
initialize
        | px py pz |

        bodies := Array new: 5.
        bodies at: 1 put: SMarkNBBody sun.
        bodies at: 2 put: SMarkNBBody jupiter.
        bodies at: 3 put: SMarkNBBody saturn.
        bodies at: 4 put: SMarkNBBody uranus.
        bodies at: 5 put: SMarkNBBody neptune.
        
        "bodies do: [:b | b print. '' println ]."

        px := 0.0.  py := 0.0.  pz := 0.0.

        bodies do: [:b |
            px := px + (b vx * b mass).
            py := py + (b vy * b mass).
            pz := pz + (b vz * b mass).
        ].
        
        (bodies at: 1) offsetMomentumX: px y: py z: pz.
        
        "bodies do: [:b | b print. '' println ]."! !

Object subclass: #SMarkPlanner
	instanceVariableNames: 'currentMark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

SMarkPlanner class
	instanceVariableNames: 'currentPlanner'!

!SMarkPlanner commentStamp: '<historical>' prior: 0!
This benchmark is an implementation of the DeltaBlue Constraint Solver described in `The DeltaBlue Algorithm: An Incremental Constraint Hierarchy Solver'', by Bjorn N. Freeman-Benson and John Maloney, Communications of the ACM, January 1990 (also as University of Washington TR 89-08-06).
!

SMarkPlanner class
	instanceVariableNames: 'currentPlanner'!

!SMarkPlanner class methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:10'!
chainTest: n
        "Do chain-of-equality-constraints performance tests."
        | vars editConstraint plan planner |
        planner := SMarkPlanner new.
        vars := Array new: n+1.
        1 to: n+1 do: [:i | vars at: i put: SMarkVariable new ].

        "thread a chain of equality constraints through the variables"
        1 to: n do:
            [ :i | | v1 v2 |
             v1 := vars at: i.
             v2 := vars at: i + 1.
             SMarkEqualityConstraint var: v1 var: v2 strength: #required].

        SMarkStayConstraint var: vars last strength: #strongDefault.
        editConstraint := SMarkEditConstraint var: (vars first) strength: #preferred.
        plan := planner extractPlanFromConstraints: (Array with: editConstraint).
        1 to: 100 do: [ :v | 
            vars first value: v.
            plan execute.
            vars last value ~= v ifTrue: [self error: 'Chain test failed!!!!']].
        editConstraint destroyConstraint
  ! !

!SMarkPlanner class methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 11:16'!
current
	^ currentPlanner! !

!SMarkPlanner class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:39'!
new
	^ currentPlanner := super new! !

!SMarkPlanner class methodsFor: 'accessing' stamp: 'tfel 5/15/2015 11:10'!
projectionTest: n
        "This test constructs a two sets of variables related to each other by
         a simple linear transformation (scale and offset)."

        | scale offset src dst planner dests |
        planner := SMarkPlanner new.
        dests := OrderedCollection new.
        scale := SMarkVariable value: 10.
        offset := SMarkVariable value: 1000.
        1 to: n do:
            [ :i |
            src := SMarkVariable value: i.
            dst := SMarkVariable value: i.
            dests add: dst.
            SMarkStayConstraint var: src strength: #default.
            SMarkScaleConstraint var: src var: scale var: offset var: dst strength: #required].

        planner changeVar: src newValue: 17.
        dst value ~= 1170 ifTrue: [self error: 'Projection test 1 failed!!!!'].

        planner changeVar: dst newValue: 1050.
        src value ~= 5 ifTrue: [self error: 'Projection test 2 failed!!!!'].

        planner changeVar: scale newValue: 5.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 1000)
                ifTrue: [self error: 'Projection test 3 failed!!!!']].

        planner changeVar: offset newValue: 2000.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 2000)
                ifTrue: [self error: 'Projection test 4 failed!!!!']]! !

!SMarkPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addConstraintsConsuming: v to: aCollection

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 (c == determiningC or: [c isSatisfied not]) ifFalse:
			[aCollection add: c]].! !

!SMarkPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addPropagate: c mark: mark
	"Recompute the walkabout strengths and stay flags of all variables
	 downstream of the given constraint and recompute the actual values
	 of all variables whose stay flag is true. If a cycle is detected,
	 remove the given constraint and answer false. Otherwise, answer true.

	 Details: Cycles are detected when a marked variable is encountered
	 downstream of the given constraint. The sender is assumed to have
	 marked the inputs of the given constraint with the given mark. Thus,
	 encountering a marked node downstream of the output constraint means
	 that there is a path from the constraint's output to one of its
	 inputs."

	| todo d |
	todo := OrderedCollection with: c.
	[todo isEmpty] whileFalse:
		[d := todo removeFirst.
		 (d output mark = mark) ifTrue:
			[self incrementalRemove: c.
			 ^ false].
		 d recalculate.
		 self addConstraintsConsuming: d output to: todo].
	^ true! !

!SMarkPlanner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:10'!
changeVar: aVariable newValue: newValue

	| editConstraint plan |
	editConstraint := SMarkEditConstraint var: aVariable strength: #preferred.
	plan := self extractPlanFromConstraints: (Array with: editConstraint).
	10 timesRepeat: [
		aVariable value: newValue.
		plan execute].
	editConstraint destroyConstraint.! !

!SMarkPlanner methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:03'!
constraintsConsuming: v do: aBlock

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 ((c == determiningC) or: [c isSatisfied not]) ifFalse:
			[aBlock value: c]].! !

!SMarkPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:59'!
extractPlanFromConstraints: constraints
	"Extract a plan for resatisfaction starting from the outputs of the
	 given constraints, usually a set of input constraints."

	| sources |
	sources := OrderedCollection new.
	constraints do:
		[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]].
	^self makePlan: sources! !

!SMarkPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:00'!
extractPlanFromVariables: variables
	"Extract a plan from the dataflow graph having the given variables. It
	 is assumed that the given set of variables is complete, or at least
	 that it contains all the input variables."

	| sources |
	sources := OrderedCollection new.
	variables do:
		[: v |
		 (v constraints) do:
			[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]]].
	^self makePlan: sources! !

!SMarkPlanner methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 15:58'!
incrementalAdd: c
	"Attempt to satisfy the given constraint and, if successful,
	 incrementally update the dataflow graph.

	 Details: If satifying the constraint is successful, it may override a
	 weaker constraint on its output. The algorithm attempts to resatisfy
	 that constraint using some other method. This process is repeated
	 until either a) it reaches a variable that was not previously
	 determined by any constraint or b) it reaches a constraint that
	 is too weak to be satisfied using any of its methods. The variables
	 of constraints that have been processed are marked with a unique mark
	 value so that we know where we've been. This allows the algorithm to
	 avoid getting into an infinite loop even if the constraint graph has
	 an inadvertent cycle."

	| mark overridden |
	mark := self newMark.
	overridden := c satisfy: mark.
	[overridden == nil] whileFalse:
		[overridden := overridden satisfy: mark].! !

!SMarkPlanner methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:34'!
incrementalRemove: c
	"Entry point for retracting a constraint. Remove the given constraint,
	 which should be satisfied, and incrementally update the dataflow
	 graph.

	 Details: Retracting the given constraint may allow some currently
	 unsatisfiable downstream constraint be satisfied. We thus collect a
	 list of unsatisfied downstream constraints and attempt to satisfy
	 each one in turn. This list is sorted by constraint strength,
	 strongest first, as a heuristic for avoiding unnecessarily adding
	 and then overriding weak constraints."

	| out unsatisfied |
	out := c output.
	c markUnsatisfied.
	c removeFromGraph.
	unsatisfied := self removePropagateFrom: out.
	unsatisfied do: [: u | self incrementalAdd: u].! !

!SMarkPlanner methodsFor: 'initialize' stamp: 'StefanMarr 4/2/2015 15:57'!
initialize

	"super initialize."

	currentMark := 1.! !

!SMarkPlanner methodsFor: 'planning' stamp: 'tfel 5/15/2015 11:08'!
makePlan: sources
	"Extract a plan for resatisfaction starting from the given satisfied
	 source constraints, usually a set of input constraints. This method
	 assumes that stay optimization is desired; the plan will contain only
	 constraints whose output variables are not stay. Constraints that do
	 no computation, such as stay and edit constraints, are not included
	 in the plan.

	 Details: The outputs of a constraint are marked when it is added to
	 the plan under construction. A constraint may be appended to the plan
	 when all its input variables are known. A variable is known if either
	 a) the variable is marked (indicating that has been computed by a
	 constraint appearing earlier in the plan), b) the variable is 'stay'
	 (i.e. it is a constant at plan execution time), or c) the variable
	 is not determined by any constraint. The last provision is for past
	 states of history variables, which are not stay but which are also
	 not computed by any constraint."

	| mark plan todo c |
	mark := self newMark.
	plan := SMarkPlan new.
	todo := sources.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 ((c output mark ~= mark) and:		"not in plan already and..."
		  [c inputsKnown: mark]) ifTrue:	"eligible for inclusion"
			[plan addLast: c.
			 c output mark: mark.
			 self addConstraintsConsuming: c output to: todo]].
	^ plan! !

!SMarkPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:37'!
newMark
	"Select a previously unused mark value.

	 Details: We just keep incrementing. If necessary, the counter will
	 turn into a LargePositiveInteger. In that case, it will be a bit
	 slower to compute the next mark but the algorithms will all behave
	 correctly. We reserve the value '0' to mean 'unmarked'. Thus, this
	 generator starts at '1' and will never produce '0' as a mark value."

	^currentMark := currentMark + 1! !

!SMarkPlanner methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:36'!
propagateFrom: v
	"The given variable has changed. Propagate new values downstream."

	| todo c |
	todo := OrderedCollection new.
	self addConstraintsConsuming: v to: todo.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 c execute.
		 self addConstraintsConsuming: c output to: todo].! !

!SMarkPlanner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:09'!
removePropagateFrom: out
	"Update the walkabout strengths and stay flags of all variables
	 downstream of the given constraint. Answer a collection of unsatisfied
	 constraints sorted in order of decreasing strength."

	| unsatisfied todo v |
	unsatisfied := OrderedCollection new.
	
	out determinedBy: nil.
	out walkStrength: SMarkPlanner absoluteWeakest.
	out stay: true.
	todo := OrderedCollection with: out.
	[todo isEmpty] whileFalse:
		[v := todo removeFirst.
		 v constraints do:
		 	[ :c | (c isSatisfied) ifFalse: [unsatisfied add: c]].
		 self constraintsConsuming: v do:
			[ :c |
			 c recalculate.
			 todo add: c output]].
	
	unsatisfied sort: [ :c1 :c2 | c1 strength stronger: c2 strength].
	^ unsatisfied! !

Object subclass: #SMarkRichObject
	instanceVariableNames: ''
	classVariableNames: 'DeviceA DeviceB DevicePacketKind HandlerA HandlerB Idler NoTask NoWork WorkPacketKind Worker'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichObject commentStamp: '<historical>' prior: 0!
This class mostly adds some constants that are used in the Richards benchmarks.!

SMarkRichObject subclass: #SMarkRichDeviceTaskDataRecord
	instanceVariableNames: 'pending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichDeviceTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that suspends itself after each time it has been run to simulate waiting for data from an external device.
!

!SMarkRichDeviceTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:40'!
create
	^ super basicNew create! !

!SMarkRichDeviceTaskDataRecord methodsFor: 'initialization' stamp: 'tfel 5/15/2015 11:30'!
create
	pending := SMarkRichObject NoWork! !

!SMarkRichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
pending
	^ pending! !

!SMarkRichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:39'!
pending: packet
	pending := packet! !

SMarkRichObject subclass: #SMarkRichHandlerTaskDataRecord
	instanceVariableNames: 'workIn deviceIn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichHandlerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets and then suspends itself.!

!SMarkRichHandlerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:03'!
create
	^ super basicNew create! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 13:42'!
asString
	^ 'HandlerTaskDataRecord(', workIn asString, ', ', deviceIn asString, ')'! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:30'!
create
	workIn := deviceIn := SMarkRichObject NoWork! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceIn
	^ deviceIn! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
deviceIn: aPacket
	deviceIn := aPacket! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceInAdd: packet
    deviceIn := self append: packet head: deviceIn! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
workIn
	^ workIn! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
workIn: aWorkQueue
	workIn := aWorkQueue! !

!SMarkRichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
workInAdd: packet
    workIn := self append: packet head: workIn! !

SMarkRichObject subclass: #SMarkRichIdleTaskDataRecord
	instanceVariableNames: 'control count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichIdleTaskDataRecord commentStamp: '<historical>' prior: 0!
An idle task doesn't do any work itself but cycles control between the two device tasks.!

!SMarkRichIdleTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:43'!
create
	^ super basicNew create! !

!SMarkRichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
control
	^ control! !

!SMarkRichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
control: aNumber
	control := aNumber! !

!SMarkRichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
count
	^ count! !

!SMarkRichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
count: aCount
	count := aCount! !

!SMarkRichIdleTaskDataRecord methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:04'!
create
    control := 1.
    count := 10000! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceA
	^ DeviceA! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceB
	^ DeviceB! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DevicePacketKind
	^ DevicePacketKind! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerA
	^ HandlerA! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerB
	^ HandlerB! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
Idler
	^ Idler! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoTask
	^ NoTask! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoWork
	^ NoWork! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
WorkPacketKind
	^ WorkPacketKind! !

!SMarkRichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
Worker
	^ Worker! !

!SMarkRichObject class methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 13:37'!
initialize
    	DeviceA := 5.
    	DeviceB := 6.
    	DevicePacketKind := 1.
    	HandlerA := 3.
    	HandlerB := 4.
    	Idler := 1.
	   NoWork := nil.
	   NoTask := nil.
    	Worker := 2.
    	WorkPacketKind := 2! !

!SMarkRichObject methodsFor: 'utilities' stamp: 'tfel 5/15/2015 11:32'!
append: packet head: queueHead
    | mouse link |
    packet link: SMarkRichObject NoWork.
    SMarkRichObject NoWork == queueHead ifTrue: [ ^ packet ].
    mouse := queueHead.
    [SMarkRichObject NoWork == (link := mouse link)]
        whileFalse: [ mouse := link ].
    mouse link: packet.
    ^ queueHead! !

SMarkRichObject subclass: #SMarkRichPacket
	instanceVariableNames: 'data datum identity kind link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichPacket commentStamp: '<historical>' prior: 0!
A simple package of data that is manipulated by the tasks.  The exact layout of the payload data carried by a packet is not importaint, and neither is the nature of the work performed on packets by the tasks. Besides carrying data, packets form linked lists and are hence used both as data and worklists.!

!SMarkRichPacket class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:44'!
create: link identity: identity kind: kind 
    ^ self basicNew
		link: link
		identity: identity
		kind: kind! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
data
	^ data! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum
	^ datum! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum: anObject
	datum := anObject! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity
	^ identity! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity: anObject
	identity := anObject! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
kind
	^ kind! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
link
	^ link! !

!SMarkRichPacket methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:45'!
link: aWorkQueue
	link := aWorkQueue! !

!SMarkRichPacket methodsFor: 'initialize' stamp: 'ClementBera 9/27/2013 15:20'!
link: aLink identity: anIdentity kind: aKind 
    link := aLink.
    identity := anIdentity. 
    kind := aKind.
    datum := 1.
    data := ByteArray new: 4! !

SMarkRichObject subclass: #SMarkRichTaskState
	instanceVariableNames: 'packetPending taskHolding taskWaiting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichTaskState commentStamp: '<historical>' prior: 0!
Abstract task that manipulates work packets.!

SMarkRichTaskState subclass: #SMarkRichTaskControlBlock
	instanceVariableNames: 'link identity priority input function handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichTaskControlBlock commentStamp: '<historical>' prior: 0!
A task control block manages a task and the queue of work packages associated with it.!

!SMarkRichTaskControlBlock class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:17'!
link: link create: identity priority: priority initialWorkQueue: initialWorkQueue initialState: initialState function: aBlock privateData: privateData 
    ^ self new
		link: link
		identity: identity
		priority: priority
		initialWorkQueue: initialWorkQueue
		initialState: initialState
		function: aBlock
		privateData: privateData! !

!SMarkRichTaskControlBlock methodsFor: 'scheduling' stamp: 'tfel 5/15/2015 11:32'!
addInput: packet checkPriority: oldTask
    SMarkRichObject NoWork == input
	ifTrue: 
	    [input := packet.
	    self packetPending: true.
	    priority > oldTask priority ifTrue: [ ^ self ]]
	ifFalse: 
	    [ input := self append: packet head: input ].
    ^ oldTask! !

!SMarkRichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
identity
	^ identity! !

!SMarkRichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
link
	^ link! !

!SMarkRichTaskControlBlock methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:16'!
link: aLink identity: anIdentity priority: aPriority initialWorkQueue: anInitialWorkQueue initialState: anInitialState function: aBlock privateData: aPrivateData 
    link := aLink.
    identity := anIdentity.
    priority := aPriority.
    input := anInitialWorkQueue.
    self packetPending: anInitialState isPacketPending.
    self taskWaiting: anInitialState isTaskWaiting.
    self taskHolding: anInitialState isTaskHolding.
    function := aBlock.
    handle := aPrivateData! !

!SMarkRichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
priority
	^ priority! !

!SMarkRichTaskControlBlock methodsFor: 'scheduling' stamp: 'tfel 5/15/2015 11:32'!
runTask
    | message |
    self isWaitingWithPacket
	ifTrue: 
	    [message := input.
	    input := message link.
	    SMarkRichObject NoWork == input 
		ifTrue: [self running]
		ifFalse: [self packetPending]]
	ifFalse: [message := SMarkRichObject NoWork].
    ^ function value: message value: handle! !

!SMarkRichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
packetPending
    ^super new packetPending! !

!SMarkRichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
running
    ^super new running! !

!SMarkRichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waiting
    ^super new waiting! !

!SMarkRichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waitingWithPacket
    ^super new waitingWithPacket! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:36'!
isPacketPending
    ^packetPending! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isRunning
    ^ packetPending not and: [taskWaiting not and: [taskHolding not]]! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskHolding
    ^taskHolding! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isTaskHoldingOrWaiting
    ^taskHolding or: [packetPending not and: [taskWaiting]]! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskWaiting
    ^taskWaiting! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaiting
    ^packetPending not and: [taskWaiting and: [taskHolding not]]! !

!SMarkRichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaitingWithPacket
    ^packetPending and: [taskWaiting and: [taskHolding not]]! !

!SMarkRichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending
    packetPending := true.
    taskWaiting := false.
    taskHolding := false! !

!SMarkRichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending: aBoolean
	packetPending := aBoolean! !

!SMarkRichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
running
    packetPending := taskWaiting := taskHolding := false! !

!SMarkRichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskHolding: aBoolean
	^ taskHolding := aBoolean! !

!SMarkRichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskWaiting: aBoolean
	^ taskWaiting := aBoolean! !

!SMarkRichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waiting
    packetPending := taskHolding := false.
    taskWaiting := true! !

!SMarkRichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waitingWithPacket
    taskHolding := false.
    taskWaiting := packetPending := true! !

SMarkRichObject subclass: #SMarkRichWorkerTaskDataRecord
	instanceVariableNames: 'destination count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichWorkerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets.!

!SMarkRichWorkerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 14:40'!
create
	^ super basicNew create! !

!SMarkRichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
count
	^ count! !

!SMarkRichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:16'!
count: anInteger
	count := anInteger! !

!SMarkRichWorkerTaskDataRecord methodsFor: 'instance creation' stamp: 'tfel 5/15/2015 11:32'!
create
    destination := SMarkRichObject HandlerA.
    count := 0! !

!SMarkRichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination
	^ destination! !

!SMarkRichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination: anObject
	destination := anObject! !

Object subclass: #SMarkRichRunner
	instanceVariableNames: 'taskList currentTask currentTaskIdentity taskTable tracing layout queuePacketCount holdCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichRunner commentStamp: '<historical>' prior: 0!
Richards simulates the task dispatcher of an operating system.!

!SMarkRichRunner class methodsFor: 'starting' stamp: 'ClementBera 9/27/2013 15:58'!
start
    "RichardsBenchmark start"

    ^self new start! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:33'!
createDevice: identity priority: priority work: work state: state 
    | data |
    data := SMarkRichDeviceTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 functionWork |
	    data1 := word.
	    functionWork := work1.
	    SMarkRichObject NoWork == functionWork 
		ifTrue:
		    [SMarkRichObject NoWork == (functionWork := data1 pending)
			ifTrue: [self wait]
			ifFalse: 
			    [data1 pending: SMarkRichObject NoWork.
			    self queuePacket: functionWork]]
		ifFalse: 
		    [data1 pending: functionWork.
			tracing ifTrue: [ self trace: functionWork datum ].
		    self holdSelf]]
	data: data! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:33'!
createHandler: identity priority: priority work: work state: state 
    | data |
    data := SMarkRichHandlerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 workPacket |
	    data1 := word.
	    SMarkRichObject NoWork == work1 
		ifFalse: [SMarkRichObject WorkPacketKind == work1 kind 
		    ifTrue: [data1 workInAdd: work1]
		    ifFalse: [data1 deviceInAdd: work1]].
	    SMarkRichObject NoWork == (workPacket := data1 workIn) 
		ifTrue: [self wait]
		ifFalse: 
		    [ | count |
			count := workPacket datum.
		    count > 4
			ifTrue: 
			    [data1 workIn: workPacket link.
			    self queuePacket: workPacket]
			ifFalse:
			    [ | devicePacket |
				SMarkRichObject NoWork == (devicePacket := data1 deviceIn) 
				ifTrue: [self wait]
				ifFalse: 
				    [data1 deviceIn: devicePacket link.
				    devicePacket datum: (workPacket data at: count).
				    workPacket datum: count + 1.
				    self queuePacket: devicePacket]]]]
	data: data! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:33'!
createIdler: identity priority: priority work: work state: state 
    | data |
    data := SMarkRichIdleTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    data1 count: data1 count - 1.
	    0 = data1 count
		ifTrue: [self holdSelf]
		ifFalse:
		    [0 = (data1 control bitAnd: 1)
			ifTrue: 
			    [data1 control: data1 control // 2.
			    self release: SMarkRichObject DeviceA]
			ifFalse: 
			    [data1 control: (data1 control // 2 bitXor: 53256).
			    self release: SMarkRichObject DeviceB]]]
	data: data! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:33'!
createPacket: link identity: identity kind: kind 
    ^ SMarkRichPacket
		create: link
		identity: identity
		kind: kind! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:34'!
createTask: identity priority: priority work: work state: state function: aBlock data: data 
    | t |
    t := SMarkRichTaskControlBlock
		link: taskList
		create: identity
		priority: priority
		initialWorkQueue: work
		initialState: state
		function: aBlock
		privateData: data.
    taskList := t.
    taskTable at: identity put: t! !

!SMarkRichRunner methodsFor: 'creation' stamp: 'tfel 5/15/2015 11:34'!
createWorker: identity priority: priority work: work state: state 
    | data |
    data := SMarkRichWorkerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    SMarkRichObject NoWork == work1
		ifTrue: [self wait]
		ifFalse: 
		    [data1 destination: (SMarkRichObject HandlerA = data1 destination
			    ifTrue:  [SMarkRichObject HandlerB]
			    ifFalse: [SMarkRichObject HandlerA]).
		    work1 identity: data1 destination.
		    work1 datum: 1.
		    1 to: 4 do: [ :i | 
			data1 count: data1 count + 1.
			data1 count > 26 ifTrue: [data1 count: 1].
			work1 data at: i put: $A asInteger + data1 count - 1].
		    self queuePacket: work1]]
	data: data! !

!SMarkRichRunner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:31'!
findTask: identity 
    | t |
    t := taskTable at: identity.
    SMarkRichObject NoTask == t ifTrue: [self error: 'findTask failed'].
    ^ t! !

!SMarkRichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:35'!
holdSelf
    holdCount := holdCount + 1.
    currentTask taskHolding: true.
    ^ currentTask link! !

!SMarkRichRunner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:31'!
initScheduler
    queuePacketCount := 0.
    holdCount := 0.
    taskTable := Array new: 6 withAll: SMarkRichObject NoTask.
    taskList := SMarkRichObject NoTask.! !

!SMarkRichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:08'!
initTrace
	tracing := false.
	layout := 0! !

!SMarkRichRunner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:31'!
queuePacket: packet 
    | t |
    t := self findTask: packet identity.
    SMarkRichObject NoTask ==  t ifTrue: [ ^ SMarkRichObject NoTask ].
    queuePacketCount := queuePacketCount + 1.
    packet link: SMarkRichObject NoWork.
    packet identity: currentTaskIdentity.
    ^ t addInput: packet checkPriority: currentTask! !

!SMarkRichRunner methodsFor: 'private' stamp: 'tfel 5/15/2015 11:31'!
release: identity 
    | t |
    t := self findTask: identity.
    SMarkRichObject NoTask == t ifTrue: [ ^ SMarkRichObject NoTask ].
    t taskHolding: false.
    t priority > currentTask priority
		ifTrue: [ ^ t ]
		ifFalse: [ ^ currentTask ]! !

!SMarkRichRunner methodsFor: 'scheduling' stamp: 'tfel 5/15/2015 11:32'!
schedule
    currentTask := taskList. 
    [SMarkRichObject NoTask == currentTask]
		whileFalse: 
	    [currentTask isTaskHoldingOrWaiting
			ifTrue: [currentTask := currentTask link]
			ifFalse: 
		    [currentTaskIdentity := currentTask identity.
			 tracing ifTrue: [ self trace: currentTaskIdentity ].
		    currentTask := currentTask runTask]]! !

!SMarkRichRunner methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:33'!
start
    | workQ |
	 self initTrace.
    self initScheduler.
    self 	createIdler: SMarkRichObject Idler
			priority: 0
			work: SMarkRichObject NoWork
			state: SMarkRichTaskState running.
    workQ := self
		createPacket: SMarkRichObject NoWork
		identity: SMarkRichObject Worker
		kind: SMarkRichObject WorkPacketKind.
    workQ := self
		createPacket: workQ
		identity: SMarkRichObject Worker
		kind: SMarkRichObject WorkPacketKind.
    self
		createWorker: SMarkRichObject Worker
		priority: 1000
		work: workQ
		state: SMarkRichTaskState waitingWithPacket.
    workQ := self
		createPacket: SMarkRichObject NoWork
		identity: SMarkRichObject DeviceA
		kind: SMarkRichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: SMarkRichObject DeviceA
		kind: SMarkRichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: SMarkRichObject DeviceA
		kind: SMarkRichObject DevicePacketKind.
    self
		createHandler: SMarkRichObject HandlerA
		priority: 2000
		work: workQ
		state: SMarkRichTaskState waitingWithPacket.
    workQ := self
		createPacket: SMarkRichObject NoWork
		identity: SMarkRichObject DeviceB
		kind: SMarkRichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: SMarkRichObject DeviceB
		kind: SMarkRichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: SMarkRichObject DeviceB
		kind: SMarkRichObject DevicePacketKind.
    self
		createHandler: SMarkRichObject HandlerB
		priority: 3000
		work: workQ
		state: SMarkRichTaskState waitingWithPacket.
    self
		createDevice: SMarkRichObject DeviceA
		priority: 4000
		work: SMarkRichObject NoWork
		state: SMarkRichTaskState waiting.
    self
		createDevice: SMarkRichObject DeviceB
		priority: 5000
		work: SMarkRichObject NoWork
		state: SMarkRichTaskState waiting.
    
	 self schedule.
    ^ (queuePacketCount = 23246) & (holdCount = 9297)
! !

!SMarkRichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:40'!
wait 
    currentTask taskWaiting: true.
    ^ currentTask! !

Object subclass: #SMarkStrength
	instanceVariableNames: 'symbolicValue arithmeticValue'
	classVariableNames: 'AbsoluteStrongest AbsoluteWeakest Required StrengthConstants StrengthTable'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkStrength commentStamp: '<historical>' prior: 0!
Strengths are used to measure the relative importance of constraints. The hierarchy of available strengths is determined by the class variable StrengthTable (see my class initialization method). Because Strengths are invariant, references to Strength instances are shared (i.e. all references to
"Strength of: #required" point to a single, shared instance). New strengths may be inserted in the strength hierarchy without disrupting current constraints.

Instance variables:
	symbolicValue		symbolic strength name (e.g. #required) <Symbol>
	arithmeticValue		index of the constraint in the hierarchy, used for comparisons <Number>!

!SMarkStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteStrongest

	^AbsoluteStrongest! !

!SMarkStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteWeakest

	^AbsoluteWeakest! !

!SMarkStrength class methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:09'!
initialize
	"Initialize the symbolic strength table. Fix the internally caches
	 values of all existing instances."
	"Strength initialize"

	StrengthTable := Dictionary new.
	StrengthTable at: #absoluteStrongest put: -10000.
	StrengthTable at: #required put: -800.
	StrengthTable at: #strongPreferred put: -600.
	StrengthTable at: #preferred put: -400.
	StrengthTable at: #strongDefault put: -200.
	StrengthTable at: #default put: 0.
	StrengthTable at: #weakDefault put: 500.
	StrengthTable at: #absoluteWeakest put: 10000.

	StrengthConstants := Dictionary new.
	StrengthTable keys do:
		[: strengthSymbol |
			StrengthConstants
				at: strengthSymbol
				put: ((super new) initializeWith: strengthSymbol)].

	AbsoluteStrongest := SMarkPlanner of: #absoluteStrongest.
	AbsoluteWeakest := SMarkPlanner of: #absoluteWeakest.
	Required := SMarkPlanner of: #required.! !

!SMarkStrength class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:52'!
of: aSymbol
	"Answer an instance with the specified strength."

	^ StrengthConstants at: aSymbol! !

!SMarkStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
required

	^Required! !

!SMarkStrength class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:39'!
strengthTable
	^ StrengthTable! !

!SMarkStrength methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:50'!
arithmeticValue
	"Answer my arithmetic value. Used for comparisons. Note that
	 STRONGER constraints have SMALLER arithmetic values."

	^arithmeticValue! !

!SMarkStrength methodsFor: 'private' stamp: 'tfel 5/15/2015 11:09'!
initializeWith: symVal
	"Record my symbolic value and reset my arithmetic value."

	symbolicValue := symVal.
	arithmeticValue := SMarkPlanner strengthTable at: symbolicValue.! !

!SMarkStrength methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:50'!
printOn: aStream
	"Append a string which represents my strength onto aStream."

	aStream nextPutAll: '%', symbolicValue, '%'.! !

!SMarkStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
sameAs: aStrength
	"Answer true if I am the same strength as the given Strength."

	^ arithmeticValue = aStrength arithmeticValue! !

!SMarkStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
stronger: aStrength
	"Answer true if I am stronger than the given Strength."

	^ arithmeticValue < aStrength arithmeticValue! !

!SMarkStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
strongest: aStrength
	"Answer the stronger of myself and aStrength."

	(aStrength stronger: self)
		ifTrue: [^aStrength]
		ifFalse: [^self]! !

!SMarkStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
weaker: aStrength
	"Answer true if I am weaker than the given Strength."

	^ arithmeticValue > aStrength arithmeticValue! !

!SMarkStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
weakest: aStrength
	"Answer the weaker of myself and aStrength."

	(aStrength weaker: self)
		ifTrue: [^aStrength]
		ifFalse: [^self].! !

Object subclass: #SMarkVariable
	instanceVariableNames: 'constraints determinedBy mark stay value walkStrength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkVariable commentStamp: '<historical>' prior: 0!
I represent a constrained variable. In addition to my value, I maintain the structure of the constraint graph, the current dataflow graph, and various parameters of interest to the DeltaBlue incremental constraint solver.

Instance variables:
	value			my value; changed by constraints, read by client <Object>
	constraints		normal constraints that reference me <Array of Constraint>
	determinedBy	the constraint that currently determines
					my value (or nil if there isn''t one) <Constraint>
	walkStrength		my walkabout strength <Strength>
	stay			true if I am a planning-time constant <Boolean>
	mark			used by the planner to mark constraints <Number>!

!SMarkVariable class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 11:01'!
value: aValue

	^self new value: aValue! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
addConstraint: aConstraint
	"Add the given constraint to the set of all constraints that refer
	 to me."

	constraints add: aConstraint.! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
constraints
	^ constraints! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy
	^ determinedBy! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy: anObject
	determinedBy := anObject! !

!SMarkVariable methodsFor: 'initialize' stamp: 'tfel 5/15/2015 11:08'!
initialize
	value := 0.
	constraints := OrderedCollection new: 2.
	determinedBy := nil.
	walkStrength := SMarkPlanner absoluteWeakest.
	stay := true.
	mark := 0.! !

!SMarkVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
longPrintOn: aStream
	^ super longPrintOn: aStream
"	self shortPrintOn: aStream.
	aStream nextPutAll: '   Constraints: '.
	(constraints isEmpty)
		ifTrue: [aStream cr; tab; nextPutAll: 'none']
		ifFalse:
			[constraints do:
				[: c | aStream cr; tab. c shortPrintOn: aStream]].
	(determinedBy isNil) ifFalse:
		[aStream cr; nextPutAll: '   Determined by: '.
		 aStream cr; tab. determinedBy shortPrintOn: aStream].
	aStream cr."! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark
	^ mark! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark: anObject
	mark := anObject! !

!SMarkVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 20:59'!
printOn: aStream
	^ super printOn: aStream
	"self shortPrintOn: aStream"! !

!SMarkVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
removeConstraint: c
	"Remove all traces of c from this variable."

	constraints remove: c ifAbsent: [].
	(determinedBy == c) ifTrue: [determinedBy := nil].! !

!SMarkVariable methodsFor: 'update' stamp: 'ClementBera 9/23/2013 10:59'!
setValue: aValue
	"Attempt to assign the given value to me using a strength of
	 #preferred."

	self setValue: aValue strength: #preferred.! !

!SMarkVariable methodsFor: 'update' stamp: 'tfel 5/15/2015 11:10'!
setValue: aValue strength: strengthSymbol
	"Attempt to assign the given value to me using the given strength."

	| editConstraint |
	editConstraint := SMarkEditConstraint var: self strength: strengthSymbol.
	(editConstraint isSatisfied) ifTrue:
		[self value: aValue.
		 SMarkPlanner propagateFrom: self].
	editConstraint destroyConstraint.! !

!SMarkVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
shortPrintOn: aStream
	^ super shortPrintOn: aStream.
"
	aStream nextPutAll: 'V(', self printString, ', '.
	aStream nextPutAll: walkStrength printString, ', '.
	(stay isNil) ifFalse:
		[aStream nextPutAll: (stay ifTrue: ['stay, '] ifFalse: ['changing, '])].
	aStream nextPutAll: value printString.
	aStream nextPutAll: ')'.
	aStream cr."! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
stay
	^ stay! !

!SMarkVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
stay: aBoolean
	stay := aBoolean! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value
	^ value! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value: anObject
	value := anObject! !

!SMarkVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
walkStrength
	^ walkStrength! !

!SMarkVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
walkStrength: aStrength
	walkStrength := aStrength! !

!Time class methodsFor: '*SMark-squeak protocol' stamp: 'sm 5/25/2011 14:26'!
milliseconds: currentTime since: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	| delta |
	delta := currentTime - lastTime.
	^ delta < 0
		ifTrue: [SmallInteger maxVal + delta]
		ifFalse: [delta]
! !

OrderedCollection subclass: #SMarkPlan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkPlan commentStamp: '<historical>' prior: 0!
A Plan is an ordered list of constraints to be executed in sequence to resatisfy all currently satisfiable constraints in the face of one or more changing inputs.!

!SMarkPlan methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:49'!
execute
	"Execute my constraints in order."

	self do: [: c | c execute].! !

Benchmark subclass: #SMarkCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfIterations
	^ 50! !

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfProcesses
	^ 8! !

!SMarkCompiler methodsFor: 'benchmarking' stamp: ''!
benchCompiler
	"was Benchmark>>testCompiler"
	SMarkCompilerTargetClass new compile:
		'somethingDummy: aBlock repeated: nTimes | i emptyBlock emptyTime blockTime |
		nTimes > 1000 ifTrue: [^self time: aBlock repeatedK: nTimes // 1000].
		emptyBlock := [].
		emptyTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [emptyBlock value]].
		blockTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [aBlock value]].
		^blockTime - emptyTime'.! !

Benchmark subclass: #SMarkDeltaBlue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkDeltaBlue commentStamp: '<historical>' prior: 0!
One-way constraint solver Benchmark. The main focus in DeltaBlue is on polymorphism and object-oriented programming

To run the benchmark, execute the expression 'SMarkDeltaBlue run'.!

!SMarkDeltaBlue methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:09'!
benchDeltaBlue
	"This the combined benchmark."
	
	| n |
	
	n := self problemSize.
	
	SMarkPlanner initialize.
	SMarkPlanner chainTest: n.
	SMarkPlanner projectionTest: n.
	^ true! !

!SMarkDeltaBlue methodsFor: 'defaults' stamp: 'ClementBera 9/23/2013 11:18'!
defaultProblemSize
	^ 5000! !

!SMarkDeltaBlue methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 11:05'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

Benchmark subclass: #SMarkFannkuch
	instanceVariableNames: 'timesRotated perm atEnd'
	classVariableNames: 'Results'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

!SMarkFannkuch class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:46'!
expectedResult: problemSize
      Results isNil ifTrue: [
        Results := Array new: 12.
        Results at:  1 put:  0.
        Results at:  2 put:  1.
        Results at:  3 put:  2.
        Results at:  4 put:  4.
        Results at:  5 put:  7.
        Results at:  6 put: 10.
        Results at:  7 put: 16.
        Results at:  8 put: 22.
        Results at:  9 put: 30.
        Results at: 10 put: 38.
        Results at: 11 put: 51.
        Results at: 12 put: 65.
      ].
      ^ Results at: problemSize! !

!SMarkFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:23'!
atEnd
        ^atEnd! !

!SMarkFannkuch methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:25'!
benchFannkuch
		| result |
      self initialize: self problemSize.
      result := self maxPfannkuchen.
      ^ result = (SMarkFannkuch expectedResult: self problemSize)! !

!SMarkFannkuch methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:25'!
benchmark
		| result |
      self initialize: self problemSize.
      result := self maxPfannkuchen.
      ^ result = (SMarkFannkuch expectedResult: self problemSize)! !

!SMarkFannkuch methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 16:31'!
initialize: size
	perm := Array new: size.
	1 to: size do: [:i | perm at: i put: i ].
	timesRotated := Array new: size withAll: 0.
	atEnd := false! !

!SMarkFannkuch methodsFor: 'benchmarking' stamp: 'tfel 4/10/2015 11:17'!
innerBenchmarkLoop
		| result |
      self initialize: innerIterations.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: innerIterations).! !

!SMarkFannkuch methodsFor: 'initialize-release' stamp: 'StefanMarr 4/3/2015 16:27'!
makeNext
	| temp remainder |
	"* Generate the next permutation. *"
	2 to: perm size do: [ :r |
		"* Rotate the first r items to the left. *"
		temp := perm at: 1.
		1 to: r - 1 do: [ :i | perm at: i put: (perm at: i + 1) ].
		perm at: r put: temp.

		timesRotated at: r put: ((timesRotated at: r) + 1) % r.
		remainder := timesRotated at: r.
		remainder = 0 ifFalse: [ ^self ].

		"* After r rotations, the first r items are in their original positions.
		 Go on rotating the first r+1 items. *"
	].

	"* We are past the final permutation. *"
	atEnd := true.! !

!SMarkFannkuch methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:23'!
maxPfannkuchen
        | max permutation |
        max := 0.
        [self atEnd] whileFalse: [
            permutation := self next.
            max := max max: (self pfannkuchen: permutation)].
        ^max! !

!SMarkFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:24'!
next
        | result |
        result := perm copy.
        self makeNext.
        ^result! !

!SMarkFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:21'!
pfannkuchen: anArray
	| first complement a b k |
	k := 0.
	[ (first := anArray at: 1) = 1 ] whileFalse: [
		k := k + 1.
		complement := first + 1.

		1 to: first / 2 do: [ :i |
			a := anArray at: i.
			b := anArray at: complement - i.
			anArray at: i put: b.
			anArray at: complement - i put: a.
	] ].
	^ k! !

Benchmark subclass: #SMarkGSGraphSearch
	instanceVariableNames: 'graphNodes graphMask updatingGraphMask graphVisited cost graphEdges k firstCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedNoOfNodes
	^ 3000000! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedTotalCost
	^ 26321966! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxInitEdges
	^ 4! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxNodes 
	^ 1 << 31! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
MaxWeight
	^ 1! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinEdges
	^ 2
! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:35'!
MinNodes
	^ 20! !

!SMarkGSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinWeight
	^ 1! !

!SMarkGSGraphSearch methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:29'!
benchGraphSearch
    | noOfNodes |
    SMarkGSJenkinsRandom seed: 49734321.
    noOfNodes := SMarkGSGraphSearch ExpectedNoOfNodes / 1000 * self problemSize.

    self initializeGraph: noOfNodes.
    self breadthFirstSearch: noOfNodes.
    ^ cost! !

!SMarkGSGraphSearch methodsFor: 'helper' stamp: 'StefanMarr 4/3/2015 15:34'!
breadthFirstSearch: noOfNodes
    | stop |
    stop := true.
    [stop] whileTrue: [
      stop := false.
      
      1 to: noOfNodes do: [:tid |
        (graphMask at: tid) ifTrue: [
          graphMask at: tid put: false.
          (graphNodes at: tid) starting 
              to: ((graphNodes at: tid) noOfEdges + (graphNodes at: tid) starting) - 1
              do: [:i |
            | id |
            id := graphEdges at: i.
            (graphVisited at: id) ifFalse: [
              cost at: id put: (cost at: tid) + 1.
              updatingGraphMask at: id put: true.
            ]
          ]
        ]
      ].
      
      1 to: noOfNodes do: [:tid |
        (updatingGraphMask at: tid) ifTrue: [
          graphMask    at: tid put: true.
          graphVisited at: tid put: true.
          stop := true.
          updatingGraphMask at: tid put: false
        ]
      ]
    ]! !

!SMarkGSGraphSearch methodsFor: 'initialization' stamp: 'tfel 5/15/2015 11:29'!
initializeGraph: noOfNodes
    | source graph totalEdges |
    graphNodes        := Array new: noOfNodes.
    graphMask         := Array new: noOfNodes withAll: false.
    updatingGraphMask := Array new: noOfNodes withAll: false.
    graphVisited      := Array new: noOfNodes withAll: false.
    cost              := Array new: noOfNodes withAll: -1.

    source := 1.

    graph := Array new: noOfNodes.
    1 to: noOfNodes do: [:i | graph at: i put: OrderedCollection new ].

    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (SMarkGSJenkinsRandom random
                       rem: SMarkGSGraphSearch MaxInitEdges - SMarkGSGraphSearch MinEdges + 1) abs
                   + SMarkGSGraphSearch MinEdges.
      1 to: noOfEdges do: [:j |
        | nodeId weight |
        nodeId := (SMarkGSJenkinsRandom random rem: noOfNodes) abs + 1.
        weight := (SMarkGSJenkinsRandom random rem: (
                    SMarkGSGraphSearch MaxWeight - SMarkGSGraphSearch MinWeight + 1)) abs
                  + SMarkGSGraphSearch MinWeight.
        (graph at: i)      append: (SMarkGSEdge newWith: nodeId and: weight).
        (graph at: nodeId) append: (SMarkGSEdge newWith:      i and: weight).
      ]
    ].

    totalEdges := 0.
    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (graph at: i) size.
      graphNodes at: i put: (SMarkGSNode newWith: totalEdges + 1 and: noOfEdges).
      totalEdges := totalEdges + noOfEdges
    ].

    graphMask    at: source put: true.
    graphVisited at: source put: true.

    graphEdges := Array new: totalEdges withAll: 0.
    
    k := 1.
    graph do: [:i |
      i do: [:j |
        graphEdges at: k put: j dest.
        k := k + 1.
      ]
    ].
    cost at: source put: 0.! !

!SMarkGSGraphSearch methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:28'!
verifyGraphSearch: result
    | totalCost |
    cost length = (SMarkGSGraphSearch ExpectedNoOfNodes / 1000 * self problemSize)
      ifFalse: [ ^ false ].

    totalCost := 0.    
    cost do: [:c | totalCost := totalCost + c].

    cost length = SMarkGSGraphSearch ExpectedNoOfNodes
      ifTrue:  [
        totalCost = SMarkGSGraphSearch ExpectedTotalCost ifFalse: [
          self error: 'ERROR: the total cost obtained for ', cost length asString,
              ' nodes is ', totalCost asString, ' while the expected cost is ',
              SMarkGSGraphSearch ExpectedTotalCost 
        ]]
      ifFalse: [
        firstCost == nil
          ifTrue: [
            firstCost := totalCost.
            ^ true ]
          ifFalse: [
            ^ firstCost = totalCost
          ]
      ].

    ^ true! !

Benchmark subclass: #SMarkJSJson
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!SMarkJSJson class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/2/2015 22:45'!
RapBenchmarkMinified
	^ '{"head":{"requestCounter":4},"operations":[["destroy","w54"],["set","w2",{"activeControl":"w99"}],["set","w21",{"customVariant":"variant_navigation"}],["set","w28",{"customVariant":"variant_selected"}],["set","w53",{"children":["w95"]}],["create","w95","rwt.widgets.Composite",{"parent":"w53","style":["NONE"],"bounds":[0,0,1008,586],"children":["w96","w97"],"tabIndex":-1,"clientArea":[0,0,1008,586]}],["create","w96","rwt.widgets.Label",{"parent":"w95","style":["NONE"],"bounds":[10,30,112,26],"tabIndex":-1,"customVariant":"variant_pageHeadline","text":"TableViewer"}],["create","w97","rwt.widgets.Composite",{"parent":"w95","style":["NONE"],"bounds":[0,61,1008,525],"children":["w98","w99","w226","w228"],"tabIndex":-1,"clientArea":[0,0,1008,525]}],["create","w98","rwt.widgets.Text",{"parent":"w97","style":["LEFT","SINGLE","BORDER"],"bounds":[10,10,988,32],"tabIndex":22,"activeKeys":["#13","#27","#40"]}],["listen","w98",{"KeyDown":true,"Modify":true}],["create","w99","rwt.widgets.Grid",{"parent":"w97","style":["SINGLE","BORDER"],"appearance":"table","indentionWidth":0,"treeColumn":-1,"markupEnabled":false}],["create","w100","rwt.widgets.ScrollBar",{"parent":"w99","style":["HORIZONTAL"]}],["create","w101","rwt.widgets.ScrollBar",{"parent":"w99","style":["VERTICAL"]}],["set","w99",{"bounds":[10,52,988,402],"children":[],"tabIndex":23,"activeKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"],"cancelKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"]}],["listen","w99",{"MouseDown":true,"MouseUp":true,"MouseDoubleClick":true,"KeyDown":true}],["set","w99",{"itemCount":118,"itemHeight":28,"itemMetrics":[[0,0,50,3,0,3,44],[1,50,50,53,0,53,44],[2,100,140,103,0,103,134],[3,240,180,243,0,243,174],[4,420,50,423,0,423,44],[5,470,50,473,0,473,44]],"columnCount":6,"headerHeight":35,"headerVisible":true,"linesVisible":true,"focusItem":"w108","selection":["w108"]}],["listen","w99",{"Selection":true,"DefaultSelection":true}],["set","w99",{"enableCellToolTip":true}],["listen","w100",{"Selection":true}],["set","w101",{"visibility":true}],["listen","w101",{"Selection":true}],["create","w102","rwt.widgets.GridColumn",{"parent":"w99","text":"Nr.","width":50,"moveable":true}],["listen","w102",{"Selection":true}],["create","w103","rwt.widgets.GridColumn",{"parent":"w99","text":"Sym.","index":1,"left":50,"width":50,"moveable":true}],["listen","w103",{"Selection":true}],["create","w104","rwt.widgets.GridColumn",{"parent":"w99","text":"Name","index":2,"left":100,"width":140,"moveable":true}],["listen","w104",{"Selection":true}],["create","w105","rwt.widgets.GridColumn",{"parent":"w99","text":"Series","index":3,"left":240,"width":180,"moveable":true}],["listen","w105",{"Selection":true}],["create","w106","rwt.widgets.GridColumn",{"parent":"w99","text":"Group","index":4,"left":420,"width":50,"moveable":true}],["listen","w106",{"Selection":true}],["create","w107","rwt.widgets.GridColumn",{"parent":"w99","text":"Period","index":5,"left":470,"width":50,"moveable":true}],["listen","w107",{"Selection":true}],["create","w108","rwt.widgets.GridItem",{"parent":"w99","index":0,"texts":["1","H","Hydrogen","Nonmetal","1","1"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w109","rwt.widgets.GridItem",{"parent":"w99","index":1,"texts":["2","He","Helium","Noble gas","18","1"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w110","rwt.widgets.GridItem",{"parent":"w99","index":2,"texts":["3","Li","Lithium","Alkali metal","1","2"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w111","rwt.widgets.GridItem",{"parent":"w99","index":3,"texts":["4","Be","Beryllium","Alkaline earth metal","2","2"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w112","rwt.widgets.GridItem",{"parent":"w99","index":4,"texts":["5","B","Boron","Metalloid","13","2"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w113","rwt.widgets.GridItem",{"parent":"w99","index":5,"texts":["6","C","Carbon","Nonmetal","14","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w114","rwt.widgets.GridItem",{"parent":"w99","index":6,"texts":["7","N","Nitrogen","Nonmetal","15","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w115","rwt.widgets.GridItem",{"parent":"w99","index":7,"texts":["8","O","Oxygen","Nonmetal","16","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w116","rwt.widgets.GridItem",{"parent":"w99","index":8,"texts":["9","F","Fluorine","Halogen","17","2"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w117","rwt.widgets.GridItem",{"parent":"w99","index":9,"texts":["10","Ne","Neon","Noble gas","18","2"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w118","rwt.widgets.GridItem",{"parent":"w99","index":10,"texts":["11","Na","Sodium","Alkali metal","1","3"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w119","rwt.widgets.GridItem",{"parent":"w99","index":11,"texts":["12","Mg","Magnesium","Alkaline earth metal","2","3"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w120","rwt.widgets.GridItem",{"parent":"w99","index":12,"texts":["13","Al","Aluminium","Poor metal","13","3"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w121","rwt.widgets.GridItem",{"parent":"w99","index":13,"texts":["14","Si","Silicon","Metalloid","14","3"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w122","rwt.widgets.GridItem",{"parent":"w99","index":14,"texts":["15","P","Phosphorus","Nonmetal","15","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w123","rwt.widgets.GridItem",{"parent":"w99","index":15,"texts":["16","S","Sulfur","Nonmetal","16","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w124","rwt.widgets.GridItem",{"parent":"w99","index":16,"texts":["17","Cl","Chlorine","Halogen","17","3"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w125","rwt.widgets.GridItem",{"parent":"w99","index":17,"texts":["18","Ar","Argon","Noble gas","18","3"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w126","rwt.widgets.GridItem",{"parent":"w99","index":18,"texts":["19","K","Potassium","Alkali metal","1","4"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w127","rwt.widgets.GridItem",{"parent":"w99","index":19,"texts":["20","Ca","Calcium","Alkaline earth metal","2","4"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w128","rwt.widgets.GridItem",{"parent":"w99","index":20,"texts":["21","Sc","Scandium","Transition metal","3","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w129","rwt.widgets.GridItem",{"parent":"w99","index":21,"texts":["22","Ti","Titanium","Transition metal","4","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w130","rwt.widgets.GridItem",{"parent":"w99","index":22,"texts":["23","V","Vanadium","Transition metal","5","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w131","rwt.widgets.GridItem",{"parent":"w99","index":23,"texts":["24","Cr","Chromium","Transition metal","6","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w132","rwt.widgets.GridItem",{"parent":"w99","index":24,"texts":["25","Mn","Manganese","Transition metal","7","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w133","rwt.widgets.GridItem",{"parent":"w99","index":25,"texts":["26","Fe","Iron","Transition metal","8","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w134","rwt.widgets.GridItem",{"parent":"w99","index":26,"texts":["27","Co","Cobalt","Transition metal","9","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w135","rwt.widgets.GridItem",{"parent":"w99","index":27,"texts":["28","Ni","Nickel","Transition metal","10","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w136","rwt.widgets.GridItem",{"parent":"w99","index":28,"texts":["29","Cu","Copper","Transition metal","11","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w137","rwt.widgets.GridItem",{"parent":"w99","index":29,"texts":["30","Zn","Zinc","Transition metal","12","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w138","rwt.widgets.GridItem",{"parent":"w99","index":30,"texts":["31","Ga","Gallium","Poor metal","13","4"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w139","rwt.widgets.GridItem",{"parent":"w99","index":31,"texts":["32","Ge","Germanium","Metalloid","14","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w140","rwt.widgets.GridItem",{"parent":"w99","index":32,"texts":["33","As","Arsenic","Metalloid","15","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w141","rwt.widgets.GridItem",{"parent":"w99","index":33,"texts":["34","Se","Selenium","Nonmetal","16","4"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w142","rwt.widgets.GridItem",{"parent":"w99","index":34,"texts":["35","Br","Bromine","Halogen","17","4"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w143","rwt.widgets.GridItem",{"parent":"w99","index":35,"texts":["36","Kr","Krypton","Noble gas","18","4"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w144","rwt.widgets.GridItem",{"parent":"w99","index":36,"texts":["37","Rb","Rubidium","Alkali metal","1","5"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w145","rwt.widgets.GridItem",{"parent":"w99","index":37,"texts":["38","Sr","Strontium","Alkaline earth metal","2","5"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w146","rwt.widgets.GridItem",{"parent":"w99","index":38,"texts":["39","Y","Yttrium","Transition metal","3","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w147","rwt.widgets.GridItem",{"parent":"w99","index":39,"texts":["40","Zr","Zirconium","Transition metal","4","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w148","rwt.widgets.GridItem",{"parent":"w99","index":40,"texts":["41","Nb","Niobium","Transition metal","5","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w149","rwt.widgets.GridItem",{"parent":"w99","index":41,"texts":["42","Mo","Molybdenum","Transition metal","6","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w150","rwt.widgets.GridItem",{"parent":"w99","index":42,"texts":["43","Tc","Technetium","Transition metal","7","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w151","rwt.widgets.GridItem",{"parent":"w99","index":43,"texts":["44","Ru","Ruthenium","Transition metal","8","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w152","rwt.widgets.GridItem",{"parent":"w99","index":44,"texts":["45","Rh","Rhodium","Transition metal","9","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w153","rwt.widgets.GridItem",{"parent":"w99","index":45,"texts":["46","Pd","Palladium","Transition metal","10","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w154","rwt.widgets.GridItem",{"parent":"w99","index":46,"texts":["47","Ag","Silver","Transition metal","11","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w155","rwt.widgets.GridItem",{"parent":"w99","index":47,"texts":["48","Cd","Cadmium","Transition metal","12","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w156","rwt.widgets.GridItem",{"parent":"w99","index":48,"texts":["49","In","Indium","Poor metal","13","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w157","rwt.widgets.GridItem",{"parent":"w99","index":49,"texts":["50","Sn","Tin","Poor metal","14","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w158","rwt.widgets.GridItem",{"parent":"w99","index":50,"texts":["51","Sb","Antimony","Metalloid","15","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w159","rwt.widgets.GridItem",{"parent":"w99","index":51,"texts":["52","Te","Tellurium","Metalloid","16","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w160","rwt.widgets.GridItem",{"parent":"w99","index":52,"texts":["53","I","Iodine","Halogen","17","5"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w161","rwt.widgets.GridItem",{"parent":"w99","index":53,"texts":["54","Xe","Xenon","Noble gas","18","5"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w162","rwt.widgets.GridItem",{"parent":"w99","index":54,"texts":["55","Cs","Caesium","Alkali metal","1","6"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w163","rwt.widgets.GridItem",{"parent":"w99","index":55,"texts":["56","Ba","Barium","Alkaline earth metal","2","6"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w164","rwt.widgets.GridItem",{"parent":"w99","index":56,"texts":["57","La","Lanthanum","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w165","rwt.widgets.GridItem",{"parent":"w99","index":57,"texts":["58","Ce","Cerium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w166","rwt.widgets.GridItem",{"parent":"w99","index":58,"texts":["59","Pr","Praseodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w167","rwt.widgets.GridItem",{"parent":"w99","index":59,"texts":["60","Nd","Neodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w168","rwt.widgets.GridItem",{"parent":"w99","index":60,"texts":["61","Pm","Promethium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w169","rwt.widgets.GridItem",{"parent":"w99","index":61,"texts":["62","Sm","Samarium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w170","rwt.widgets.GridItem",{"parent":"w99","index":62,"texts":["63","Eu","Europium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w171","rwt.widgets.GridItem",{"parent":"w99","index":63,"texts":["64","Gd","Gadolinium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w172","rwt.widgets.GridItem",{"parent":"w99","index":64,"texts":["65","Tb","Terbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w173","rwt.widgets.GridItem",{"parent":"w99","index":65,"texts":["66","Dy","Dysprosium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w174","rwt.widgets.GridItem",{"parent":"w99","index":66,"texts":["67","Ho","Holmium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w175","rwt.widgets.GridItem",{"parent":"w99","index":67,"texts":["68","Er","Erbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w176","rwt.widgets.GridItem",{"parent":"w99","index":68,"texts":["69","Tm","Thulium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w177","rwt.widgets.GridItem",{"parent":"w99","index":69,"texts":["70","Yb","Ytterbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w178","rwt.widgets.GridItem",{"parent":"w99","index":70,"texts":["71","Lu","Lutetium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w179","rwt.widgets.GridItem",{"parent":"w99","index":71,"texts":["72","Hf","Hafnium","Transition metal","4","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w180","rwt.widgets.GridItem",{"parent":"w99","index":72,"texts":["73","Ta","Tantalum","Transition metal","5","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w181","rwt.widgets.GridItem",{"parent":"w99","index":73,"texts":["74","W","Tungsten","Transition metal","6","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w182","rwt.widgets.GridItem",{"parent":"w99","index":74,"texts":["75","Re","Rhenium","Transition metal","7","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w183","rwt.widgets.GridItem",{"parent":"w99","index":75,"texts":["76","Os","Osmium","Transition metal","8","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w184","rwt.widgets.GridItem",{"parent":"w99","index":76,"texts":["77","Ir","Iridium","Transition metal","9","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w185","rwt.widgets.GridItem",{"parent":"w99","index":77,"texts":["78","Pt","Platinum","Transition metal","10","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w186","rwt.widgets.GridItem",{"parent":"w99","index":78,"texts":["79","Au","Gold","Transition metal","11","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w187","rwt.widgets.GridItem",{"parent":"w99","index":79,"texts":["80","Hg","Mercury","Transition metal","12","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w188","rwt.widgets.GridItem",{"parent":"w99","index":80,"texts":["81","Tl","Thallium","Poor metal","13","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w189","rwt.widgets.GridItem",{"parent":"w99","index":81,"texts":["82","Pb","Lead","Poor metal","14","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w190","rwt.widgets.GridItem",{"parent":"w99","index":82,"texts":["83","Bi","Bismuth","Poor metal","15","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w191","rwt.widgets.GridItem",{"parent":"w99","index":83,"texts":["84","Po","Polonium","Metalloid","16","6"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w192","rwt.widgets.GridItem",{"parent":"w99","index":84,"texts":["85","At","Astatine","Halogen","17","6"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w193","rwt.widgets.GridItem",{"parent":"w99","index":85,"texts":["86","Rn","Radon","Noble gas","18","6"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w194","rwt.widgets.GridItem",{"parent":"w99","index":86,"texts":["87","Fr","Francium","Alkali metal","1","7"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w195","rwt.widgets.GridItem",{"parent":"w99","index":87,"texts":["88","Ra","Radium","Alkaline earth metal","2","7"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w196","rwt.widgets.GridItem",{"parent":"w99","index":88,"texts":["89","Ac","Actinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w197","rwt.widgets.GridItem",{"parent":"w99","index":89,"texts":["90","Th","Thorium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w198","rwt.widgets.GridItem",{"parent":"w99","index":90,"texts":["91","Pa","Protactinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w199","rwt.widgets.GridItem",{"parent":"w99","index":91,"texts":["92","U","Uranium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w200","rwt.widgets.GridItem",{"parent":"w99","index":92,"texts":["93","Np","Neptunium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w201","rwt.widgets.GridItem",{"parent":"w99","index":93,"texts":["94","Pu","Plutonium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w202","rwt.widgets.GridItem",{"parent":"w99","index":94,"texts":["95","Am","Americium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w203","rwt.widgets.GridItem",{"parent":"w99","index":95,"texts":["96","Cm","Curium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w204","rwt.widgets.GridItem",{"parent":"w99","index":96,"texts":["97","Bk","Berkelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w205","rwt.widgets.GridItem",{"parent":"w99","index":97,"texts":["98","Cf","Californium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w206","rwt.widgets.GridItem",{"parent":"w99","index":98,"texts":["99","Es","Einsteinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w207","rwt.widgets.GridItem",{"parent":"w99","index":99,"texts":["100","Fm","Fermium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w208","rwt.widgets.GridItem",{"parent":"w99","index":100,"texts":["101","Md","Mendelevium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w209","rwt.widgets.GridItem",{"parent":"w99","index":101,"texts":["102","No","Nobelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w210","rwt.widgets.GridItem",{"parent":"w99","index":102,"texts":["103","Lr","Lawrencium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w211","rwt.widgets.GridItem",{"parent":"w99","index":103,"texts":["104","Rf","Rutherfordium","Transition metal","4","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w212","rwt.widgets.GridItem",{"parent":"w99","index":104,"texts":["105","Db","Dubnium","Transition metal","5","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w213","rwt.widgets.GridItem",{"parent":"w99","index":105,"texts":["106","Sg","Seaborgium","Transition metal","6","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w214","rwt.widgets.GridItem",{"parent":"w99","index":106,"texts":["107","Bh","Bohrium","Transition metal","7","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w215","rwt.widgets.GridItem",{"parent":"w99","index":107,"texts":["108","Hs","Hassium","Transition metal","8","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w216","rwt.widgets.GridItem",{"parent":"w99","index":108,"texts":["109","Mt","Meitnerium","Transition metal","9","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w217","rwt.widgets.GridItem",{"parent":"w99","index":109,"texts":["110","Ds","Darmstadtium","Transition metal","10","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w218","rwt.widgets.GridItem",{"parent":"w99","index":110,"texts":["111","Rg","Roentgenium","Transition metal","11","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w219","rwt.widgets.GridItem",{"parent":"w99","index":111,"texts":["112","Uub","Ununbium","Transition metal","12","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w220","rwt.widgets.GridItem",{"parent":"w99","index":112,"texts":["113","Uut","Ununtrium","Poor metal","13","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w221","rwt.widgets.GridItem",{"parent":"w99","index":113,"texts":["114","Uuq","Ununquadium","Poor metal","14","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w222","rwt.widgets.GridItem",{"parent":"w99","index":114,"texts":["115","Uup","Ununpentium","Poor metal","15","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w223","rwt.widgets.GridItem",{"parent":"w99","index":115,"texts":["116","Uuh","Ununhexium","Poor metal","16","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w224","rwt.widgets.GridItem",{"parent":"w99","index":116,"texts":["117","Uus","Ununseptium","Halogen","17","7"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w225","rwt.widgets.GridItem",{"parent":"w99","index":117,"texts":["118","Uuo","Ununoctium","Noble gas","18","7"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w226","rwt.widgets.Composite",{"parent":"w97","style":["BORDER"],"bounds":[10,464,988,25],"children":["w227"],"tabIndex":-1,"clientArea":[0,0,986,23]}],["create","w227","rwt.widgets.Label",{"parent":"w226","style":["NONE"],"bounds":[10,10,966,3],"tabIndex":-1,"text":"Hydrogen (H)"}],["create","w228","rwt.widgets.Label",{"parent":"w97","style":["WRAP"],"bounds":[10,499,988,16],"tabIndex":-1,"foreground":[150,150,150,255],"font":[["Verdana","Lucida Sans","Arial","Helvetica","sans-serif"],10,false,false],"text":"Shortcuts: [CTRL+F] - Filter | Sort by: [CTRL+R] - Number, [CTRL+Y] - Symbol, [CTRL+N] - Name, [CTRL+S] - Series, [CTRL+G] - Group, [CTRL+E] - Period"}],["set","w1",{"focusControl":"w99"}],["call","rwt.client.BrowserNavigation","addToHistory",{"entries":[["tableviewer","TableViewer"]]}]]}'
    ! !

!SMarkJSJson methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:35'!
benchJson
	^ (SMarkJSJsonParser with: SMarkJSJson RapBenchmarkMinified) parse! !

!SMarkJSJson methodsFor: 'benchmarking' stamp: 'tfel 5/15/2015 11:37'!
verifyJson: result
	result class = SMarkJSParseException ifTrue: [ result println ].
	result isObject ifFalse: [^ false].
	(result asObject at: 'head') isObject ifFalse: [^ false].
	(result asObject at: 'operations') isArray ifFalse: [^ false].
	^ (result asObject at: 'operations') asArray size = 156! !

Benchmark subclass: #SMarkLoops
	instanceVariableNames: 'someInstVar'
	classVariableNames: 'ClassVarValue'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkLoops commentStamp: '<historical>' prior: 0!
SMarkLoops implements a set of microbenchmarks measuring a number of basic aspects such as message send, instance field access, and array access cost.

Example use:
  (SMarkLoops new runOnly: #benchFloatLoop) run: 10 !

!SMarkLoops class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/10/2012 16:56'!
defaultProblemSize
	^ 50000! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 9/12/2012 13:23'!
benchArrayAccess
	| i obj array |
	
	array := Array new: 1.
	obj   := Object new.
	
	i := self problemSize.
	[i > 0] whileTrue: [
		array at: 1.
		array at: 1 put: obj.
		
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:55'!
benchClassVarBinding
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		r := ClassVarValue.
		ClassVarValue := nil.

		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchFloatLoop
	| a limit |
	a := 1.23.
	limit := self problemSize.
	[a < limit] whileTrue: [
		a := a + 0.6.
		a := a + 0.4.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchInstVarAccess
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		r := someInstVar.
		someInstVar := nil.

		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchIntLoop
	| i |
	i := self problemSize.
	[i > 0] whileTrue: [
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchSend
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		self doNothing.
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchSendWithManyArguments
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		self doNothingA: i b: i c: i d: i e: i f: i g: i h: i i: i j: i.
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'helper' stamp: 'StefanMarr 1/10/2012 10:32'!
doNothing! !

!SMarkLoops methodsFor: 'helper' stamp: 'StefanMarr 1/10/2012 10:35'!
doNothingA: p1 b: p2 c: p3 d: p4 e: p5 f: p6 g: p7 h: p8 i: p9 j: p10! !

Benchmark subclass: #SMarkMandelbrot
	instanceVariableNames: 'firstResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

!SMarkMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:47'!
benchmark
	^ self mandelbrot: self problemSize! !

!SMarkMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:46'!
mandelbrot: size
        | sum byteAcc bitNum y |
        sum     := 0.
        byteAcc := 0.
        bitNum  := 0.
  
        y := 0.
        
        [y < size] whileTrue: [
            | ci x |
            ci := (2.0 * y / size) - 1.0.
            x  := 0.
            
            [x < size] whileTrue: [
                | zr zrzr zi zizi cr escape z notDone |
                zrzr := zr := 0.0.
                zizi := zi := 0.0.
                cr   := (2.0 * x / size) - 1.5.
                
                z := 0.
                notDone := true.
                escape := 0.
                [notDone and: [z < 50]] whileTrue: [
                    zr := zrzr - zizi + cr.
                    zi := 2.0 * zr * zi + ci.
            
                    "preserve recalculation"
                    zrzr := zr * zr.
                    zizi := zi * zi.
            
                    (zrzr + zizi > 4.0) ifTrue: [
                        notDone := false.
                        escape  := 1.
                    ].
                    z := z + 1.
                ].
  
                byteAcc := (byteAcc << 1) + escape.
                bitNum  := bitNum + 1.
  
                " Code is very similar for these cases, but using separate blocks
                  ensures we skip the shifting when it's unnecessary,
                  which is most cases. "
                bitNum = 8 
                    ifTrue: [
                      sum := sum bitXor: byteAcc.
                      byteAcc := 0.
                      bitNum  := 0. ]
                    ifFalse: [
                      (x = (size - 1)) ifTrue: [
                          byteAcc := byteAcc << (8 - bitNum).
                          sum := sum bitXor: byteAcc.
                          byteAcc := 0.
                          bitNum  := 0. ]].
                x := x + 1.
            ].
            y := y + 1.
        ].
        
        ^ sum! !

!SMarkMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:44'!
verifymark: result
	  self problemSize = 750 ifTrue: [
        ^ result = 50
      ].
      firstResult isNil ifTrue: [
        firstResult := result.
      ].
      ^ result = firstResult! !

Benchmark subclass: #SMarkNBody
	instanceVariableNames: 'expectedEnergy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!SMarkNBody methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 11:30'!
benchmark
	     | bodies |
        bodies := SMarkNBodySystem new.
        
        1 to: self problemSize do: [:i |
            bodies advance: 0.01.
        ].
        
        self problemSize = 250000 ifTrue: [
          ^ bodies energy = -0.1690859889909308
        ].
        expectedEnergy isNil ifTrue: [
          expectedEnergy := bodies energy.
          ^ true
        ].
        
        ^ expectedEnergy = bodies energy.! !

!SMarkNBody methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 11:29'!
initialize
	SMarkNBBody initialize! !

Benchmark subclass: #SMarkRSqueak
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-RSqueak'!

!SMarkRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:57'!
benchBitBlt

	#( paint over ) do: [:rule|
		| destRect depths |
		destRect := 0@0 extent: 600@600.
		depths := #( 1 2 4 8 16 32 ).
		depths do:[:destDepth| | dest |
			depths do:[:sourceDepth| | bb source |
				dest := Form extent: destRect extent depth: destDepth.
				source := Form extent: destRect extent depth: sourceDepth.
				(source getCanvas) fillOval: dest boundingBox color: Color yellow borderWidth: 30 borderColor: Color black.
				bb := (WarpBlt toForm: dest)
					sourceForm: source;
					sourceRect: source boundingBox;
					destRect: dest boundingBox;
					colorMap: (source colormapIfNeededFor: dest);
					combinationRule: (Form perform: rule).
				self problemSize timesRepeat: [bb copyBits]]]].! !

!SMarkRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchDSAGen

	| randomString |
	randomString := '4647912 April 2015''unix'''.
	self problemSize timesRepeat: [
		[DigitalSignatureAlgorithm new
			initRandomFromString: randomString;
			generateKeySet] timeToRun].! !

!SMarkRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchRenderFont
	| height ttc glyphs |
	height := (TextStyle pointsToPixels: (12 * 1.2)) rounded.
	ttc := (TextStyle named: 'BitstreamVeraSerif') fontArray last ttcDescription.
	glyphs := (((1 to: ttc size - 1) collect: [:i | ttc at: i]) reject: [:g | g = ttc fallbackGlyph or: [g contours size = 0]]) first: 52.
	self problemSize timesRepeat: [
		glyphs do: [:g| 
			g asFormWithScale: (height asFloat / ttc fontHeight)
					ascender: ttc ascender 
					descender: ttc descender 
					fgColor: Color black bgColor: Color transparent depth: 32]].! !

!SMarkRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchShaLongString

	| longString veryLongString |
	longString := (PackageInfo named: 'Help-Squeak-Project') postscript contents asString.
	veryLongString := String streamContents: [:s| 5 timesRepeat: [s nextPutAll: longString]].
	self problemSize timesRepeat: [SecureHashAlgorithm new hashMessage: veryLongString].! !

Benchmark subclass: #SMarkRichards
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichards commentStamp: '<historical>' prior: 0!
Richards is an OS kernel simulation benchmark, originally written in BCPL by Martin Richards. The main focus in Richards is on property access and calling functions and methods.!

!SMarkRichards methodsFor: 'benchs' stamp: 'tfel 5/15/2015 11:32'!
benchRichards

	SMarkRichObject initialize.
	^ SMarkRichRunner start! !

!SMarkRichards methodsFor: 'benchs' stamp: 'ClementBera 9/27/2013 15:56'!
defaultProblemSize
	^ 50! !

!SMarkRichards methodsFor: 'benchs' stamp: 'ClementBera 9/27/2013 15:56'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkRichards methodsFor: 'benchs' stamp: 'StefanMarr 4/3/2015 14:12'!
verifyRichards: result
	^ result! !

Benchmark subclass: #SMarkSlopstone
	instanceVariableNames: 'obj o'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkSlopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
I added one class methods.
and modified execute method.  block variables are renamed for Squeak.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

original comments
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

SUMMARY The filein includes two classes: Slopstones (Smalltalk Low level
OPeration Stones) and Smopstones (Smalltalk Medium level OPeration Stones).
Each includes seven cpu intensive benchmarks. They work equally well with
16-bit and 32-bit implementations and are designed to be portable to all
Smalltalk versions from ParcPlace and Digitalk. They are normalized to the
author's 486/33 Windows 3.1 machine running ParcPlace VisualWorks 1.0.
Results have been posted to the Usenet group comp.lang.smalltalk and form the
basis of an article that is scheduled to be published in the June issue of The
Smalltalk Report.

The only difference between the ST80 (STones80) and ST/V (STonesV) filein is
in the messages that define the classes in the first few lines of code. The
ST80 messages specify the class category and message protocol, which are not
used in ST/V.

Bruce Samuelson

!

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/7/2013 17:16'!
benchStone
	"Using the SlopStone benchmarks, but not doing the
	 old style Stone performance number reporting"

	| n |
	n := self problemSize.
	n > SmallInteger maxVal
		ifTrue: [ self error: 'Count exceeded max small int.' ].
	
	o := obj new.
	
	1 to: n do: [:i | self doIntAdd         ].
	1 to: n do: [:i | self doFloatAdd       ].
	1 to: n do: [:i | self doStringAccess   ].
	1 to: n do: [:i | self doObjectCreation ].
	1 to: n do: [:i | self doObjectCopy     ].
	1 to: n do: [:i | self doPerform        ].
	1 to: n do: [:i | self doBlockValue     ].
! !

!SMarkSlopstone methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ 1! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:52'!
doBlockValue
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:48'!
doFloatAdd
	1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+
	1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:47'!
doIntAdd
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:16'!
doObjectCopy
	o copy copy copy copy copy copy copy copy copy copy! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:11'!
doObjectCreation
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:17'!
doPerform
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:50'!
doStringAccess
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 9/9/2012 02:16'!
readme
   "STEFAN: use the original benchmarks, but do not use original way
            of reporting results. Rely on the SMark reporting instead."

	"INTRODUCTION 
	 
	Slopstone: Smalltalk Low level OPeration Stones 
	Portable Low Level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallIntegers) Placed in public domain January 1993  (c) Bruce 
	Samuelson Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SlopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	  
	The benchmarks test strictly low level operations. They do not test 
	higher level operations such as forming sets, sorting, or streaming, nor 
	do they test 
	applications. They also do not test user interface operations because of 
	the non-portability of this area of Smalltalk and its sensitivity to the 
	performance of the video subsystem. The tests are cpu bound. They do 
	not access files and should not cause disk paging. 
	 
	The benchmarks use loop counts of 16000 because SmallIntegers cannot 
	exceed 16383 for ST/V-DOS. 16-bit implementions would perform worse 
	with large loop 
	counts. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SLOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one slopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	1000s    time    1000s of 
	itera-   sec-    iterations   slop- 
	tions    onds    per sec      stones   explanation 
	 
	3808     0.577   6600         1.0      add integers 
	 544     2.262    240         1.0      add floats 
	 960     1.088    882         1.0      access strings 
	 320     0.908    352         1.0      create objects 
	 160     1.49     107         1.0      copy objects 
	 480     1.129    425         1.0      perform selectors 
	 896     1.237    724         1.0      evaluate blocks 
	 
	 640     1.151    555         1.0      geometric mean"
	^ self! !

!SMarkSlopstone methodsFor: 'initialize-release' stamp: 'StefanMarr 5/16/2013 12:26'!
setUp
	obj := Object.! !

Benchmark subclass: #SMarkSmopstone
	instanceVariableNames: 'ordCollectionFact intervalFact rectFactory sortedColFact arrFact setFact pointFact'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkSmopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
see the comment of SlopstoneBenchmark class.
I added all class methods and initialization>initilize method,
modified:
	streamTestsOn: method.  In some case use testPrintOn: s base: instead of printOn:,
	primesUpTo: method. From recursively calling a block to whileTrue: block.
	sorcerersApprentice method. From recursively calling a block to  recursive method.
	execute and setup method.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

originak code by
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

!

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/16/2013 11:43'!
benchStone
	| n primes strings set |
	
	n := self problemSize.
	
	self fractonacci: n / 13. "13/2"
	
	primes := self primesUpTo: n * 100. "7351 - 1" "9000"
	self streamTestsOn: primes.
	
	strings := self stringsUpTo: n * 100. "8000"
	set := self setFrom: strings.
	self sort: set.
	
	self sorcerersApprentice: n.! !

!SMarkSmopstone methodsFor: 'defaults' stamp: 'StefanMarr 5/16/2013 11:44'!
defaultProblemSize
	^ 80! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'nishis 2/14/98 02:47'!
fractonacci: n 
	"Return something like the fibonacci function of n but
	using fractional numbers rather than whole ones. The
	reason for this variation is to run long enough to get
	a decent time measurement without exceeding 16383, the
	limit of small integers for ST/V-DOS. Choosing n = 13/2
	takes enough time and computes to 13581.

	Fibonacci uses n-1 and n-2 instead of n-(1/2) and n-(1/3).
	However, I couldn't get it to run in the above constraints.

	This benchmark tests the efficiency of recursively calling
	a method that does a little fractional arithmetic internally."

	^n > 1
		ifTrue: [(self fractonacci: n - (1/2)) + (self fractonacci: n - (1/3))]
		ifFalse: [1]! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:10'!
primesUpTo: n
	"modified by nishis"

	"Return the prime numbers between 2 and n.

	This method tests the efficiency of recursively calling a block
	that does some collection enumeration based on integer arithmetic."

	| nSqrt lowPrimes highPrimes genNext first |
	n < 5 | (n > 16363) ifTrue: [self error: 'Prime limit(s) out of range.'].
	nSqrt := n sqrt rounded.
	lowPrimes := ordCollectionFact with: 2.
	highPrimes := intervalFact from: 5 to: n by: 2.
	
	genNext := [:nextPrime |
		lowPrimes add: nextPrime.
		highPrimes := highPrimes select: [:k | k \\ nextPrime ~= 0].
		(first := highPrimes first) <= nSqrt ifTrue: [genNext value: first]].
	
	genNext value: 3.

	^ lowPrimes , highPrimes! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 9/9/2012 02:50'!
readme
   "STEFAN: use the original benchmarks, but do not use original way
            of reporting results. Rely on the SMark reporting instead."

	"INTRODUCTION 
	 
	Smopstone: Smalltalk Medium level OPeration Stones 
	Portable Medium level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallInts) Placed in public domain January 1993  (c) Bruce Samuelson 
	Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SmopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	 
	These benchmarks are a companion to the SlopstoneBenchmark class 
	posted to 
	comp.lang.smalltalk this month. Slopstones tested low level operations. 
	  
	Smopstones test medium level operations that exercise recursive block 
	and  
	method calls, collection building and enumeration, streaming, and 
	sorting. The 
	lower level operations contained in them exercise arithmetic (mostly 
	integer, with some fractions and floats) string manipulation, and low 
	level streaming. 
	 
	The benchmarks do not test applications. They also do not test user 
	interface performance because of the non-portability of this area of 
	Smalltalk and its  
	sensitivity to the speed of the video subsystem. The tests are cpu bound. 
	They do not access files and should not cause disk paging. 
	 
	The main weaknesses of the benchmarks are (1) they are not high 
	enough level 
	to test actual applications, and (2) they concentrate in too few areas of 
	Smalltalk, omitting many of the diverse capabilities of its class library. 
	My excuse is that one can only devote limited time writing public 
	domain benchmarks. 
	 
	The tests avoid generating integers larger than 16383, the maximum 
	SmallInteger in ST/V-DOS. 16-bit implementions would perform worse 
	with larger 
	integers. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. They try to avoid other pitfalls that would skew the results 
	such as the lack of an adequate hash function for a class. Someone 
	warned of this 
	in comp.lang.smalltalk (I forget who). 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SMOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one smopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	time in    smop- 
	seconds    stones    explanation 
	 
	 
	3.157      1.0       generating fractonaccis 
	1.123      1.0       generating primes 
	1.091      1.0       generating and parsing streams 
	3.091      1.0       generating strings 
	1.167      1.0       forming sets 
	5.139      1.0       sorting strings 
	5.601      1.0       sorcerer's apprentice 
	 
	2.355      1.0       geometric mean"
	^ self! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 19:05'!
setFrom: collection
	"Form a set from collection and return it.

	This method tests the efficiency of building a fairly large set
	from strings. It indirectly tests the effectiveness of the string
	hash function. Strings are used often enough as dictionary keys
	that this may be worth including in the benchmark suite.  ST/V-DOS
	has a primitive hash for strings, and ST80 has an elaborate one
	written in Smalltalk."

	^ setFact withAll: collection! !

!SMarkSmopstone methodsFor: 'running' stamp: 'StefanMarr 5/7/2013 19:27'!
setUp
	ordCollectionFact := OrderedCollection.
	intervalFact      := Interval.
	rectFactory       := Rectangle.
	sortedColFact     := SortedCollection.
	arrFact           := Array.
	setFact           := Set.
	pointFact         := Point.! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/16/2013 11:43'!
sorcerersApprentice: m
	"modified by nishis"

	" FORMATTED FOR MONOSPACED FONT

	Perform various operations on rectangles.

	This method tests the efficiency of recursively calling a block that
	includes lots of integer arithmetic, collection building, and collection
	enumeration. The method:

	(1) Creates a collection of pseudo random rectangles 
	(2) Forms a new collection of all their intersections 
	(3) Recursively continues until there are no more intersections 
	(4) Returns a collection with the counts of rectangles in each generation.

	Because the intersections are forming progressively smaller rectangles
	(we exclude intersections of a rectangle with itself), the algorithm will
	eventually converge. Depending on the choice of numeric parameters, it may
	converge very quickly or very slowly. The parameters used below make it
	converge in a reasonable amount of time (a few seconds on a one-smopstone
	machine). It took some experimentation with different combinations to
	achieve this.

	The pseudo random number generator isn't very good, but it's adequate
	for this benchmark. I had intended the number '87' it uses to be a prime,
	but 87 = 29 * 3. The numbers may have been a bit more random otherwise.

	One could write an algorithm that would converge much more quickly and in
	a more predictable amount of time by sorting the intermediate rectangles
	in two dimensions and not bothering to test for intersections those
	rectangles that are contained in mutually exclusive regions. We have
	chosen algorithmic simplicity over performance optimization. We simply
	perform intersections of each rectangle with every possible partner in
	each generation. The time consumed is quadratic in the number of rectangles.

	The algorithm originally stored rectangles in sets to eliminate duplicates.
	Unfortunately, ST/V-DOS uses the hash function inherited from Object for
	Rectangle, which will allow duplicates to be stored. So we were forced to
	store rectangles in ordered collections and eliminate duplicates by brute
	force. The brutality was heightened because we could not use the test
	collection>>includes: to decide whether to add a rectangle to the ordered
	collections, since ST/V-DOS does not define equality (=) for rectangles
	either. The remaining warts in the code are not worth explaining.

	In an actual application, these shortcomings of ST/V-DOS would have been
	overcome by adding subclasses and methods rather than writing kludgy code."

	| n firstGen intersection isIncluded counts r random
	a b c d e f g h generate nextGen |
	
	n := 20 * m.
	firstGen := ordCollectionFact new.
	counts   := ordCollectionFact new.
	r := 50.
	random := [r := r + 1 * 87 \\ n].
	m timesRepeat: [
		a := random value.
		b := random value.
		c := random value.
		d := random value.
		e := a min: b.
		f := c min: d.
		g := a max: b.
		h := c max: d.
		firstGen add: (rectFactory origin: (pointFact x: e y: f) corner: (pointFact x: g y: h))].
	generate := 
		[:lastGen |
		counts add: lastGen size.
		nextGen := ordCollectionFact new.
		lastGen do:
			[:r1 |
			lastGen do: 
				[:r2 | 
				(r1 origin ~= r2 origin or: [r1 corner ~= r2 corner])
				"In ST80 this test would have simply been r1 ~= r2"
					ifTrue:
						[(r1 intersects: r2)
							ifTrue:
								[intersection := r1 intersect: r2.
								isIncluded := false.                     "All these lines"
								nextGen do:                              "would have been"
									[:rec |                                "avoided if we"
										(rec origin = intersection origin and: "could have used"
										[rec corner = intersection corner])    "a set for"
											ifTrue: [isIncluded := true]].       "nextGen. See"
								isIncluded                               "explanation"
									ifFalse:                               "above."
										[nextGen size > 500
											ifTrue: [self error: 'Converges too slowly.']
											ifFalse: [nextGen add: intersection]]]]]].
			nextGen size > 0 ifTrue: [generate value: nextGen]].
	
	generate value: firstGen.
	^counts! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:18'!
sort: collection
	"Form a sorted collection from collection and return it.

	This method tests the efficiency of sorting a fairly large
	collection of strings. It indirectly measures the efficiency
	of the sorting algorithm and of string comparison operations."

	^collection as: sortedColFact! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:19'!
streamTestsOn: integers 
	"modified by nishis"
	"Test steaming operations on the collection of integers. 
	 
	This method measures the efficiency of integer-to-float conversion, of 
	printing numbers to a write stream, of parsing tokens in a read 
	stream,   and of converting the tokens from strings to numbers. The 
	technique for 
	converting tokens into floats is constrained by portability between 
	ST80 and ST/V. 
	 
	To validate the logic, the original integers are compared with the 
	final   floats. There should be no roundoff errors."
	"The following line accounts for the different implementations of 
	Float>>printString for some versions of Smalltalk. USA versions use 
	the decimal character, while some European versions use the comma 
	char.   Thanks to Marten Feldtmann for pointing this out."

	| delim space s floats float string |
	delim := 1.0 printString at: 2.	"$. for USA, $, for some Europe."

	space := Character value: 32.
	"Can't use Character space in ST/V-DOS"
	s := ReadWriteStream on: String new.
	integers do: 
		[:i |
			i asFloat printOn: s.
			s space].
	"Now make sure the underlying string size is < 16383, a 16-bit small int."
	s contents size > (SmallInteger maxVal) ifTrue: [self error: 'String too big.'].
	s reset.
	floats := ordCollectionFact new: integers size.
	[s atEnd]
		whileFalse: 
			[float := 0.0.
			string := s upTo: delim.
			s upTo: space.
			"In the following, digitValue is portable between ST80 and ST/V-DOS. "
			string do: [:char | float := float * 10.0 + char digitValue].
			floats add: float].
	integers = floats ifFalse: [self error: 'Numbers do not compare.']! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 20:18'!
stringsUpTo: n 
	"Return a collection of strings representing the integers from 1 
	to n with their digits reversed. 
	 
	This method tests the efficiency of creating small streams, performing 
	string operations, and building collections. It includes a gross kludge 
	to coerce portability between ST80 and ST/V. They vary slightly in the 
	selector used to reverse collections. Subsequent to freezing this code, 
	someone pointed out that it could have been written more simply 
	without   using perform:. It could also have been simplified using 
	select:. It 
	was too late to change. The techniques are left as an exercise for the 
	reader."

	| selector |
	(arrFact with: #reverse with: #reversed)
		do: [:symbol | (String canUnderstand: symbol)
			ifTrue: [selector := symbol]].
	^ (intervalFact from: 1 to: n)
		collect: [:m | m printString perform: selector]! !

Benchmark subclass: #SMarkSqueakJS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SqueakJS'!

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:36'!
benchmarkBitBltExampleOne

	self bltExampleOneTimes: self problemSize extent: 32@32.
	self bltExampleOneTimes: self problemSize extent: 64@64.
	self bltExampleOneTimes: self problemSize extent: 256@256.! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:33'!
benchmarkFib
	
	self problemSize benchFib.! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:52'!
benchmarkFillArray
	| b o |
	b := Array new: self problemSize.
	o := Object new.
	self problemSize timesRepeat: [b atAllPut: o].! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:51'!
benchmarkFillByteArray
	| b |
	b := ByteArray new: self problemSize.
	self problemSize timesRepeat: [b atAllPut: 11].! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:51'!
benchmarkFillString
	| b |
	b := String new: self problemSize.
	self problemSize timesRepeat: [b atAllPut: $a].! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:42'!
benchmarkIntegerBench

	self problemSize benchmark.! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:31'!
benchmarkMandala
	
	| f npoints d l pen points |
	npoints := self problemSize.
	f := Form extent: 1000@1000 depth: 32.
	pen := Pen newOnForm: f.
	d := f height - 50.
	l := 3.14 * d / npoints.
	pen home; up; turn: -90; go: d // 2; turn: 90; go: 0 - l / 2; down.
	points := Array new: npoints.
	1 to: npoints do: 
		[:i | 
		points at: i put: pen location rounded.
		pen go: l; turn: 360.0 / npoints].
	npoints // 2
		to: 1
		by: -1
		do: 
			[:i | 
			pen color: i.
			1 to: npoints do: 
				[:j | 
				pen place: (points at: j).
				pen goto: (points at: j + i - 1 \\ npoints + 1)]]
! !

!SMarkSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:41'!
bltExampleOneTimes: n extent: aPoint

	| s d aBitBlt |
	s := Form extent: aPoint.
	s fillWhite.
	'Starting and Quitting
Like most Smalltalk''s, the machine-executable portion is a relatively small program known as the "virtual machine" (VM).  The VM''s job is to provide services from the physical machine to real Smalltalk objects.  Services like input and output.  The Smalltalk system, including all data and code, is a system of objects, built from the ground up, and interpreted by this virtual computer.  This affords Smalltalk system platform portability.' asParagraph
		wrappingBox: (10@10 corner: 600@600)
		clippingBox: (0@0 extent: aPoint);
		displayOn: s at: 0@0.
	s fillBlack: (7@7 corner: 25@25).
	s fillBlack: (39@39 corner: 47@47).
	d := Form extent: aPoint.
	aBitBlt := BitBlt
				destForm: d
				sourceForm: s
				fillColor: nil
				combinationRule: 1
				destOrigin: 0@0
				sourceOrigin: 0@0
				extent: s extent
				clipRect: (0@0 corner: aPoint).
	1 to: n do: [:i |
		aBitBlt combinationRule: i \\ 16.
		d fillWhite.
		d fillBlack: (0@0 corner: 64@16).
		aBitBlt copyBits].
	! !
SMarkJSJsonLiteral initialize!
SMarkNBBody initialize!
SMarkRichObject initialize!
SMarkStrength initialize!
